#!/usr/bin/env python
#
# Copyright 2011-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, filter, blocks


class test_dc_blocker(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        ''' Test impulse response - long form, cc '''
        src_data = [1, ] + 100 * [0, ]
        expected_result = (
            (-0.02072429656982422 + 0j),
            (-0.02081298828125 + 0j),
            (0.979156494140625 + 0j),
            (-0.02081298828125 + 0j),
            (-0.02072429656982422 + 0j))

        src = blocks.vector_source_c(src_data)
        op = filter.dc_blocker_cc(32, True)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op, dst)
        self.tb.run()

        # only test samples around 2D-2
        result_data = dst.data()[60:65]
        self.assertComplexTuplesAlmostEqual(expected_result, result_data)

    def test_002(self):
        ''' Test impulse response - short form, cc '''
        src_data = [1, ] + 100 * [0, ]
        expected_result = ((-0.029296875 + 0j), (-0.0302734375 + 0j),
                           (0.96875 + 0j), (-0.0302734375 + 0j),
                           (-0.029296875 + 0j))

        src = blocks.vector_source_c(src_data)
        op = filter.dc_blocker_cc(32, False)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op, dst)
        self.tb.run()

        # only test samples around D-1
        result_data = dst.data()[29:34]
        self.assertComplexTuplesAlmostEqual(expected_result, result_data)

    def test_003(self):
        ''' Test impulse response - long form, ff '''
        src_data = [1, ] + 100 * [0, ]
        expected_result = ((-0.02072429656982422), (-0.02081298828125),
                           (0.979156494140625), (-0.02081298828125),
                           (-0.02072429656982422))

        src = blocks.vector_source_f(src_data)
        op = filter.dc_blocker_ff(32, True)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        # only test samples around 2D-2
        result_data = dst.data()[60:65]
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)

    def test_004(self):
        ''' Test impulse response - short form, ff '''
        src_data = [1, ] + 100 * [0, ]
        expected_result = ((-0.029296875), (-0.0302734375),
                           (0.96875), (-0.0302734375),
                           (-0.029296875))

        src = blocks.vector_source_f(src_data)
        op = filter.dc_blocker_ff(32, False)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        # only test samples around D-1
        result_data = dst.data()[29:34]
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_dc_blocker)



#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

###############################################################################
# Imports
###############################################################################
from argparse import ArgumentParser
from gnuradio.eng_arg import eng_float, intx
import gui
import sys
import os
from PyQt5 import Qt, QtGui, QtCore, uic
import PyQt5.Qwt5 as Qwt
from gnuradio import zeromq
import signal


class gui(QtGui.QMainWindow):
    def __init__(self, window_name, options, parent=None):
        QtGui.QMainWindow.__init__(self, parent)

        # give Ctrl+C back to system
        signal.signal(signal.SIGINT, signal.SIG_DFL)

        self.gui = uic.loadUi(os.path.join(
            os.path.dirname(__file__), 'main_window.ui'), self)

        self.update_timer = Qt.QTimer()

        # socket addresses
        rpc_adr_server = "tcp://" + options.servername + ":6666"
        rpc_adr_client = "tcp://" + options.clientname + ":6667"
        probe_adr_server = "tcp://" + options.servername + ":5556"
        probe_adr_client = "tcp://" + options.clientname + ":5557"

        # ZeroMQ
        self.probe_manager = zeromq.probe_manager()
        self.probe_manager.add_socket(
            probe_adr_server, 'float32', self.plot_data_server)
        self.probe_manager.add_socket(
            probe_adr_client, 'float32', self.plot_data_client)

        self.rpc_mgr_server = zeromq.rpc_manager()
        self.rpc_mgr_server.set_request_socket(rpc_adr_server)
        self.rpc_mgr_client = zeromq.rpc_manager()
        self.rpc_mgr_client.set_request_socket(rpc_adr_client)

        self.gui.setWindowTitle(window_name)
        self.gui.qwtPlotServer.setTitle("Signal Scope")
        self.gui.qwtPlotServer.setAxisTitle(Qwt.QwtPlot.xBottom, "Samples")
        self.gui.qwtPlotServer.setAxisTitle(Qwt.QwtPlot.yLeft, "Amplitude")
        self.gui.qwtPlotServer.setAxisScale(Qwt.QwtPlot.xBottom, 0, 100)
        self.gui.qwtPlotServer.setAxisScale(Qwt.QwtPlot.yLeft, -2, 2)
        self.gui.qwtPlotClient.setTitle("Signal Scope")
        self.gui.qwtPlotClient.setAxisTitle(Qwt.QwtPlot.xBottom, "Samples")
        self.gui.qwtPlotClient.setAxisTitle(Qwt.QwtPlot.yLeft, "Amplitude")
        self.gui.qwtPlotClient.setAxisScale(Qwt.QwtPlot.xBottom, 0, 100)
        self.gui.qwtPlotClient.setAxisScale(Qwt.QwtPlot.yLeft, -2, 2)

        # Grid
        pen = Qt.QPen(Qt.Qt.DotLine)
        pen.setColor(Qt.Qt.black)
        pen.setWidth(0)
        grid_server = Qwt.QwtPlotGrid()
        grid_client = Qwt.QwtPlotGrid()
        grid_server.setPen(pen)
        grid_client.setPen(pen)
        grid_server.attach(self.gui.qwtPlotServer)
        grid_client.attach(self.gui.qwtPlotClient)

        # Signals
        self.connect(self.update_timer, QtCore.SIGNAL(
            "timeout()"), self.probe_manager.watcher)
        self.connect(self.gui.pushButtonRunServer, QtCore.SIGNAL(
            "clicked()"), self.start_fg_server)
        self.connect(self.gui.pushButtonStopServer,
                     QtCore.SIGNAL("clicked()"), self.stop_fg_server)
        self.connect(self.gui.pushButtonRunClient, QtCore.SIGNAL(
            "clicked()"), self.start_fg_client)
        self.connect(self.gui.pushButtonStopClient,
                     QtCore.SIGNAL("clicked()"), self.stop_fg_client)
        self.connect(self.gui.comboBox, QtCore.SIGNAL(
            "currentIndexChanged(QString)"), self.set_waveform)
        self.connect(self.gui.spinBox, QtCore.SIGNAL(
            "valueChanged(int)"), self.set_gain)
        self.shortcut_start = QtGui.QShortcut(
            Qt.QKeySequence("Ctrl+S"), self.gui)
        self.shortcut_stop = QtGui.QShortcut(
            Qt.QKeySequence("Ctrl+C"), self.gui)
        self.shortcut_exit = QtGui.QShortcut(
            Qt.QKeySequence("Ctrl+D"), self.gui)
        self.connect(self.shortcut_exit, QtCore.SIGNAL(
            "activated()"), self.gui.close)

        # start update timer
        self.update_timer.start(30)

    def start_fg_server(self):
        self.rpc_mgr_server.request("start_fg")

    def stop_fg_server(self):
        self.rpc_mgr_server.request("stop_fg")

    def start_fg_client(self):
        self.rpc_mgr_client.request("start_fg")

    def stop_fg_client(self):
        self.rpc_mgr_client.request("stop_fg")

    # plot the data from the queues
    def plot_data(self, plot, samples):
        self.x = list(range(0, len(samples), 1))
        self.y = samples
        # clear the previous points from the plot
        plot.clear()
        # draw curve with new points and plot
        curve = Qwt.QwtPlotCurve()
        curve.setPen(Qt.QPen(Qt.Qt.blue, 2))
        curve.attach(plot)
        curve.setData(self.x, self.y)
        plot.replot()

    def plot_data_server(self, samples):
        self.plot_data(self.gui.qwtPlotServer, samples)

    def plot_data_client(self, samples):
        self.plot_data(self.gui.qwtPlotClient, samples)

    def set_waveform(self, waveform_str):
        self.rpc_mgr_server.request("set_waveform", [str(waveform_str)])

    def set_gain(self, gain):
        self.rpc_set_gain(gain)

    def rpc_set_gain(self, gain):
        self.rpc_mgr_server.request("set_k", [gain])

###############################################################################
# Options Parser
###############################################################################


def parse_args():
    """Options parser."""
    parser = ArgumentParser()
    parser.add_argument("-s", "--servername", default="localhost",
                        help="Server hostname")
    parser.add_argument("-c", "--clientname", default="localhost",
                        help="Server hostname")
    args = parser.parse_args()
    return args


###############################################################################
# Main
###############################################################################
if __name__ == "__main__":
    args = parse_args()
    qapp = Qt.QApplication(sys.argv)
    qapp.main_window = gui("Remote GNU Radio GUI", args)
    qapp.main_window.show()
    qapp.exec_()



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt
import math


class test_tags_strobe(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        N = 10000
        nsamps = 1000
        ntags = N / nsamps - 1

        src = blocks.tags_strobe(gr.sizeof_float,
                                 pmt.intern("TEST"), nsamps)
        hed = blocks.head(gr.sizeof_float, N)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, hed, dst)
        self.tb.run()

        self.assertEqual(ntags, len(dst.tags()))

        n_expected = nsamps
        for tag in dst.tags():
            self.assertEqual(tag.offset, n_expected)
            n_expected += nsamps

    def test_002(self):
        N = 10000
        nsamps = 123
        ntags = N // nsamps

        src = blocks.tags_strobe(gr.sizeof_float,
                                 pmt.intern("TEST"), nsamps)
        hed = blocks.head(gr.sizeof_float, N)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, hed, dst)
        self.tb.run()

        self.assertEqual(ntags, len(dst.tags()))

        n_expected = nsamps
        for tag in dst.tags():
            self.assertEqual(tag.offset, n_expected)
            n_expected += nsamps

    def test_003(self):
        N = 100000
        nsamps = 10000
        ntags = N / nsamps - 1

        src = blocks.tags_strobe(gr.sizeof_float,
                                 pmt.intern("TEST"), nsamps)
        hed = blocks.head(gr.sizeof_float, N)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, hed, dst)
        self.tb.run()

        self.assertEqual(ntags, len(dst.tags()))

        n_expected = nsamps
        for tag in dst.tags():
            self.assertEqual(tag.offset, n_expected)
            n_expected += nsamps


if __name__ == '__main__':
    gr_unittest.run(test_tags_strobe)



#!/usr/bin/env python
#
# Copyright 2006,2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, video_sdl


class test_video_sdl (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_nop(self):
        """Just see if we can import the module...
        They may not have video drivers, etc.  Don't try to run anything"""
        pass


if __name__ == '__main__':
    gr_unittest.run(test_video_sdl)




#
# Copyright 2011-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import numpy

import pmt

from gnuradio import gr, gr_unittest, blocks


class non_sync_block(gr.basic_block):
    def __init__(self):
        gr.basic_block.__init__(self,
                                name="non_sync_block",
                                in_sig=[numpy.float32],
                                out_sig=[numpy.float32, numpy.float32])

    def general_work(self, input_items, output_items):
        self.consume(0, len(input_items[0]))
        self.produce(0, 2)
        self.produce(1, 1)
        return gr.WORK_CALLED_PRODUCE


class add_2_f32_1_f32(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="add 2 f32",
            in_sig=[numpy.float32, numpy.float32],
            out_sig=[numpy.float32],
        )

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0] + input_items[1]
        return len(output_items[0])


class add_2_fc32_1_fc32(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="add 2 fc32",
            in_sig=[numpy.complex64, numpy.complex64],
            out_sig=[numpy.complex64],
        )

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0] + input_items[1]
        return len(output_items[0])


class convolve(gr.sync_block):
    """
    A demonstration using block history to properly perform a convolution.
    """

    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="convolve",
            in_sig=[numpy.float32],
            out_sig=[numpy.float32]
        )
        self._taps = [1, 0, 0, 0]
        self.set_history(len(self._taps))

    def work(self, input_items, output_items):
        output_items[0][:] = numpy.convolve(
            input_items[0], self._taps, mode='valid')
        return len(output_items[0])


class decim2x(gr.decim_block):
    def __init__(self):
        gr.decim_block.__init__(
            self,
            name="decim2x",
            in_sig=[numpy.float32],
            out_sig=[numpy.float32],
            decim=2
        )

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0][::2]
        return len(output_items[0])


class interp2x(gr.interp_block):
    def __init__(self):
        gr.interp_block.__init__(
            self,
            name="interp2x",
            in_sig=[numpy.float32],
            out_sig=[numpy.float32],
            interp=2
        )

    def work(self, input_items, output_items):
        output_items[0][1::2] = input_items[0]
        output_items[0][::2] = input_items[0]
        return len(output_items[0])


class tag_source(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="tag source",
            in_sig=None,
            out_sig=[numpy.float32],
        )

    def work(self, input_items, output_items):
        num_output_items = len(output_items[0])

        # put code here to fill the output items...
        if self.nitems_written(0) == 0:
            # skip tagging in the first work block
            return num_output_items

        # make a new tag on the middle element every time work is called
        count = self.nitems_written(0) + num_output_items // 2
        key = pmt.string_to_symbol("example_key")
        value = pmt.string_to_symbol("example_value")
        self.add_item_tag(0, count, key, value)

        return num_output_items


class tag_sink(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="tag sink",
            in_sig=[numpy.float32],
            out_sig=None,
        )
        self.key = None

    def work(self, input_items, output_items):
        num_input_items = len(input_items[0])

        # put code here to process the input items...

        # print all the tags received in this work call
        nread = self.nitems_read(0)
        tags = self.get_tags_in_range(0, nread, nread + num_input_items)
        for tag in tags:
            # print tag.offset
            # print pmt.symbol_to_string(tag.key)
            # print pmt.symbol_to_string(tag.value)
            self.key = pmt.symbol_to_string(tag.key)

        return num_input_items


class tag_sink_win(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(self, name="tag sink",
                               in_sig=[numpy.float32],
                               out_sig=None)
        self.key = None

    def work(self, input_items, output_items):
        num_input_items = len(input_items[0])
        tags = self.get_tags_in_window(0, 0, num_input_items)
        for tag in tags:
            self.key = pmt.symbol_to_string(tag.key)
        return num_input_items


class fc32_to_f32_2(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="fc32_to_f32_2",
            in_sig=[numpy.complex64],
            out_sig=[(numpy.float32, 2)],
        )

    def work(self, input_items, output_items):
        output_items[0][::, 0] = numpy.real(input_items[0])
        output_items[0][::, 1] = numpy.imag(input_items[0])
        return len(output_items[0])


class vector_to_stream(gr.interp_block):
    def __init__(self, itemsize, nitems_per_block):
        gr.interp_block.__init__(
            self,
            name="vector_to_stream",
            in_sig=[(itemsize, nitems_per_block)],
            out_sig=[itemsize],
            interp=nitems_per_block
        )
        self.block_size = nitems_per_block

    def work(self, input_items, output_items):
        n = 0
        for i in range(len(input_items[0])):
            for j in range(self.block_size):
                output_items[0][n] = input_items[0][i][j]
                n += 1

        return len(output_items[0])


class test_block_gateway(gr_unittest.TestCase):

    def test_add_f32(self):
        tb = gr.top_block()
        src0 = blocks.vector_source_f([1, 3, 5, 7, 9], False)
        src1 = blocks.vector_source_f([0, 2, 4, 6, 8], False)
        adder = add_2_f32_1_f32()
        adder.name()
        sink = blocks.vector_sink_f()
        tb.connect((src0, 0), (adder, 0))
        tb.connect((src1, 0), (adder, 1))
        tb.connect(adder, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 5, 9, 13, 17])

    def test_add_fc32(self):
        tb = gr.top_block()
        src0 = blocks.vector_source_c([1, 3j, 5, 7j, 9], False)
        src1 = blocks.vector_source_c([0, 2j, 4, 6j, 8], False)
        adder = add_2_fc32_1_fc32()
        sink = blocks.vector_sink_c()
        tb.connect((src0, 0), (adder, 0))
        tb.connect((src1, 0), (adder, 1))
        tb.connect(adder, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 5j, 9, 13j, 17])

    def test_convolve(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1, 2, 3, 4, 5, 6, 7, 8], False)
        cv = convolve()
        sink = blocks.vector_sink_f()
        tb.connect(src, cv, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 2, 3, 4, 5, 6, 7, 8])

    def test_decim2x(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1, 2, 3, 4, 5, 6, 7, 8], False)
        d2x = decim2x()
        sink = blocks.vector_sink_f()
        tb.connect(src, d2x, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 3, 5, 7])

    def test_interp2x(self):
        tb = gr.top_block()
        src = blocks.vector_source_f([1, 3, 5, 7, 9], False)
        i2x = interp2x()
        sink = blocks.vector_sink_f()
        tb.connect(src, i2x, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 1, 3, 3, 5, 5, 7, 7, 9, 9])

    def test_tags(self):
        src = tag_source()
        sink = tag_sink()
        # should be enough items to get a tag through
        head = blocks.head(gr.sizeof_float, 50000)
        tb = gr.top_block()
        tb.connect(src, head, sink)
        tb.run()
        self.assertEqual(sink.key, "example_key")

    def test_tags_win(self):
        src = tag_source()
        sink = tag_sink_win()
        # should be enough items to get a tag through
        head = blocks.head(gr.sizeof_float, 50000)
        tb = gr.top_block()
        tb.connect(src, head, sink)
        tb.run()
        self.assertEqual(sink.key, "example_key")

    def test_fc32_to_f32_2(self):
        tb = gr.top_block()
        src = blocks.vector_source_c(
            [1 + 2j, 3 + 4j, 5 + 6j, 7 + 8j, 9 + 10j], False)
        convert = fc32_to_f32_2()
        v2s = vector_to_stream(numpy.float32, 2)
        sink = blocks.vector_sink_f()
        tb.connect(src, convert, v2s, sink)
        tb.run()
        self.assertEqual(sink.data(), [1, 2, 3, 4, 5, 6, 7, 8, 9, 10])

    def test_non_sync_block(self):
        tb = gr.top_block()
        src = blocks.vector_source_f(range(1000000))
        sinks = [blocks.vector_sink_f(), blocks.vector_sink_f()]
        dut = non_sync_block()
        tb.connect(src, dut)
        tb.connect((dut, 0), sinks[0])
        tb.connect((dut, 1), sinks[1])
        tb.run()
        self.assertEqual(len(sinks[0].data()), 2 * len(sinks[1].data()))


if __name__ == '__main__':
    gr_unittest.run(test_block_gateway)



#!/usr/bin/env python3
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Unit tests for fft.window
"""

import numpy
from gnuradio import gr_unittest
from gnuradio import fft


class test_window(gr_unittest.TestCase):
    """
    Unit tests for fft.window
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_normwin(self):
        """
        Verify window normalization
        """
        win = fft.window.build(
            fft.window.WIN_BLACKMAN_hARRIS,
            21,
            normalize=True)
        power = numpy.sum([x * x for x in win]) / len(win)
        self.assertAlmostEqual(power, 1.0, places=6)


if __name__ == '__main__':
    gr_unittest.run(test_window)



#!/usr/bin/env python
# Copyright 2012, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, digital, blocks
import pmt


class qa_digital_carrier_allocator_cvc (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "ts_last"

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        """
        pretty simple (the carrier allocation here is not a practical OFDM configuration!)
        """
        fft_len = 6
        tx_symbols = (1, 2, 3)
        #             ^ this gets mapped to the DC carrier because occupied_carriers[0][0] == 0
        pilot_symbols = ((1j,),)
        occupied_carriers = ((0, 1, 2),)
        pilot_carriers = ((3,),)
        sync_word = (list(range(fft_len)),)
        expected_result = sync_word[0] + [1j, 0, 0, 1, 2, 3]
        #                                                 ^ DC carrier
        src = blocks.vector_source_c(tx_symbols, False, 1)
        alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                   occupied_carriers,
                                                   pilot_carriers,
                                                   pilot_symbols, sync_word,
                                                   self.tsb_key)
        sink = blocks.tsb_vector_sink_c(vlen=fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_symbols),
                self.tsb_key),
            alloc,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_001_t2(self):
        """
        pretty simple (same as before, but odd fft len)
        """
        fft_len = 5
        tx_symbols = (1, 2, 3)
        #             ^ this gets mapped to the DC carrier because occupied_carriers[0][0] == 0
        occupied_carriers = ((0, 1, 2),)
        pilot_carriers = ((-2,),)
        pilot_symbols = ((1j,),)
        expected_result = [1j, 0, 1, 2, 3]
        #                         ^ DC carrier
        src = blocks.vector_source_c(tx_symbols, False, 1)
        alloc = digital.ofdm_carrier_allocator_cvc(
            fft_len,
            occupied_carriers,
            pilot_carriers,
            pilot_symbols, (),
            self.tsb_key
        )
        sink = blocks.tsb_vector_sink_c(vlen=fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_symbols),
                self.tsb_key),
            alloc,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_002_t(self):
        """
        same, but using negative carrier indices
        """
        fft_len = 6
        tx_symbols = (1, 2, 3)
        pilot_symbols = ((1j,),)
        occupied_carriers = ((-1, 1, 2),)
        pilot_carriers = ((3,),)
        expected_result = [1j, 0, 1, 0, 2, 3]
        src = blocks.vector_source_c(tx_symbols, False, 1)
        alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                   occupied_carriers,
                                                   pilot_carriers,
                                                   pilot_symbols, (),
                                                   self.tsb_key)
        sink = blocks.tsb_vector_sink_c(fft_len)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_symbols),
                self.tsb_key),
            alloc,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)

    def test_002_tb(self):
        """
        once again, but this time add a sync word
        """
        fft_len = 6
        sync_word = (0,) * fft_len
        tx_symbols = (1, 2, 3, 4, 5, 6)
        pilot_symbols = ((1j,),)
        occupied_carriers = ((-1, 1, 2),)
        pilot_carriers = ((3,),)
        expected_result = list(
            sync_word + (1j, 0, 1, 0, 2, 3) + (1j, 0, 4, 0, 5, 6))
        special_tag1 = gr.tag_t()
        special_tag1.offset = 0
        special_tag1.key = pmt.string_to_symbol("spam")
        special_tag1.value = pmt.to_pmt(23)
        special_tag2 = gr.tag_t()
        special_tag2.offset = 4
        special_tag2.key = pmt.string_to_symbol("eggs")
        special_tag2.value = pmt.to_pmt(42)
        src = blocks.vector_source_c(
            tx_symbols, False, 1,
            (special_tag1, special_tag2)
        )
        alloc = digital.ofdm_carrier_allocator_cvc(
            fft_len,
            occupied_carriers,
            pilot_carriers,
            pilot_symbols,
            sync_words=(sync_word,),
            len_tag_key=self.tsb_key
        )
        sink = blocks.tsb_vector_sink_c(fft_len)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_symbols),
                self.tsb_key),
            alloc,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)
        tags = [gr.tag_to_python(x) for x in sink.tags()]
        tags = sorted([(x.offset, x.key, x.value) for x in tags])
        tags_expected = [
            (0, 'spam', 23),
            (2, 'eggs', 42),
        ]
        self.assertEqual(tags, tags_expected)

    def test_003_t(self):
        """
        more advanced:
        - 6 symbols per carrier
        - 2 pilots per carrier
        - have enough data for nearly 3 OFDM symbols
        - send that twice
        - add some random tags
        - don't shift
        """
        tx_symbols = list(range(1, 16))  # 15 symbols
        pilot_symbols = ((1j, 2j), (3j, 4j))
        occupied_carriers = ((1, 3, 4, 11, 12, 14), (1, 2, 4, 11, 13, 14),)
        pilot_carriers = ((2, 13), (3, 12))
        expected_result = list(
            (0,
             1,
             1j,
             2,
             3,
             0,
             0,
             0,
             0,
             0,
             0,
             4,
             5,
             2j,
             6,
             0,
             0,
             7,
             8,
             3j,
             9,
             0,
             0,
             0,
             0,
             0,
             0,
             10,
             4j,
             11,
             12,
             0,
             0,
             13,
             1j,
             14,
             15,
             0,
             0,
             0,
             0,
             0,
             0,
             0,
             0,
             2j,
             0,
             0))
        fft_len = 16
        testtag1 = gr.tag_t()
        testtag1.offset = 0
        testtag1.key = pmt.string_to_symbol('tag1')
        testtag1.value = pmt.from_long(0)
        testtag2 = gr.tag_t()
        testtag2.offset = 7  # On the 2nd OFDM symbol
        testtag2.key = pmt.string_to_symbol('tag2')
        testtag2.value = pmt.from_long(0)
        testtag3 = gr.tag_t()
        testtag3.offset = len(tx_symbols) + 1  # First OFDM symbol of packet 2
        testtag3.key = pmt.string_to_symbol('tag3')
        testtag3.value = pmt.from_long(0)
        testtag4 = gr.tag_t()
        # Last OFDM symbol of packet 2
        testtag4.offset = 2 * len(tx_symbols) - 1
        testtag4.key = pmt.string_to_symbol('tag4')
        testtag4.value = pmt.from_long(0)
        src = blocks.vector_source_c(
            tx_symbols * 2, False, 1, (testtag1, testtag2, testtag3, testtag4))
        alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                   occupied_carriers,
                                                   pilot_carriers,
                                                   pilot_symbols, (),
                                                   self.tsb_key,
                                                   False)
        sink = blocks.tsb_vector_sink_c(fft_len)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                1,
                len(tx_symbols),
                self.tsb_key),
            alloc,
            sink)
        self.tb.run()
        self.assertEqual(sink.data()[0], expected_result)
        tags_found = {
            'tag1': False,
            'tag2': False,
            'tag3': False,
            'tag4': False}
        correct_offsets = {'tag1': 0, 'tag2': 1, 'tag3': 3, 'tag4': 5}
        for tag in sink.tags():
            key = pmt.symbol_to_string(tag.key)
            if key in list(tags_found.keys()):
                tags_found[key] = True
                self.assertEqual(correct_offsets[key], tag.offset)
        self.assertTrue(all(tags_found.values()))

    def test_004_t(self):
        """
        Provoking TypeError exceptions providing wrong user input (earlier invisible SIGFPE).
        """
        fft_len = 6

        # Occupied carriers
        # with self.assertRaises(TypeError) as oc:
        # Pybind11 raises this as a ValueError
        with self.assertRaises(ValueError) as oc:
            alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                       [],
                                                       [[], ],
                                                       [[], ],
                                                       [],
                                                       self.tsb_key)

        # Pilot carriers
        # Pybind11 raises this as a ValueError
        with self.assertRaises(ValueError) as pc:
            alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                       [[], ],
                                                       [],
                                                       [[], ],
                                                       [],
                                                       self.tsb_key)

        # Pilot carrier symbols
        # Pybind11 raises this as a ValueError
        with self.assertRaises(ValueError) as ps:
            alloc = digital.ofdm_carrier_allocator_cvc(fft_len,
                                                       [[], ],
                                                       [[], ],
                                                       [],
                                                       [],
                                                       self.tsb_key)

        self.assertEqual(str(
            oc.exception), "Occupied carriers must be of type vector of vector i.e. ((),).")
        self.assertEqual(str(pc.exception),
                         "Pilot carriers must be of type vector of vector i.e. ((),).")
        self.assertEqual(str(ps.exception),
                         "Pilot symbols must be of type vector of vector i.e. ((),).")


if __name__ == '__main__':
    gr_unittest.run(qa_digital_carrier_allocator_cvc)



#!/usr/bin/env python
#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, blocks
from multiprocessing import Process
import numpy


# This block just exists to pass data through and throw an exception
class except_block(gr.sync_block):
    def __init__(self, throw_except, except_count=10000):
        gr.sync_block.__init__(
            self,
            name="except_block",
            in_sig=[numpy.complex64],
            out_sig=[numpy.complex64],
        )
        self.throw_except = throw_except
        self.except_count = except_count
        self.count = 0

    def work(self, input_items, output_items):
        output_items[0][:] = input_items[0]
        self.count += len(output_items[0])
        if self.count >= self.except_count:
            raise RuntimeError("Error in except_block")
        return len(output_items[0])


def process_func(catch_exceptions):
    tb = gr.top_block(catch_exceptions=catch_exceptions)
    # some test data
    src_data = [complex(x, x + 1) for x in range(65536)]
    src = blocks.vector_source_c(src_data)
    src.set_repeat(True)

    e_block_1 = except_block(False)
    e_block_2 = except_block(True)

    sink_1 = blocks.null_sink(gr.sizeof_gr_complex)
    sink_2 = blocks.null_sink(gr.sizeof_gr_complex)

    tb.connect(src, e_block_1)
    tb.connect(src, e_block_2)
    tb.connect(e_block_1, sink_1)
    tb.connect(e_block_2, sink_2)
    tb.run()


class test_uncaught_exception(gr_unittest.TestCase):

    def test_exception_throw_uncaught(self):
        # Test to ensure that throwing an exception causes the
        # process running top_block to exit
        p = Process(target=process_func, args=(False,))
        p.daemon = True
        p.start()
        p.join(10.0)
        exit_code = p.exitcode
        self.assertIsNotNone(
            exit_code, "exception did not cause flowgraph exit")

    def test_exception_throw_caught(self):
        # Test to ensure that throwing an exception does not cause the process
        # running top_block to exit (in catch_exceptions mode)
        p = Process(target=process_func, args=(True,))
        p.daemon = True
        p.start()
        p.join(2.5)
        exit_code = p.exitcode
        self.assertIsNone(
            exit_code, "exception caused flowgraph exit")


if __name__ == '__main__':
    gr_unittest.run(test_uncaught_exception)



"""
Copyright 2008-2015 Free Software Foundation, Inc.
This file is part of GNU Radio

SPDX-License-Identifier: GPL-2.0-or-later

"""


import sys
import re
import subprocess
import threading
import json
import random
import itertools
import queue


###############################################################################
# The docstring extraction
###############################################################################

def docstring_guess_from_key(key):
    """
    Extract the documentation from the python __doc__ strings
    By guessing module and constructor names from key

    Args:
        key: the block key

    Returns:
        a dict (block_name --> doc string)
    """
    doc_strings = dict()

    in_tree = [key.partition('_')[::2] + (
        lambda package: getattr(__import__('gnuradio.' + package), package),
    )]

    key_parts = key.split('_')
    oot = [
        ('_'.join(key_parts[:i]), '_'.join(key_parts[i:]), __import__)
        for i in range(1, len(key_parts))
    ]

    for module_name, init_name, importer in itertools.chain(in_tree, oot):
        if not module_name or not init_name:
            continue
        try:
            module = importer(module_name)
            break
        except ImportError:
            continue
    else:
        return doc_strings

    pattern = re.compile(get_block_regex(init_name))

    for match in filter(pattern.match, dir(module)):
        try:
            doc_strings[match] = getattr(module, match).__doc__
        except AttributeError:
            continue

    return doc_strings


def get_block_regex(block_name):
    """
    Helper function to create regular expression for a given block name
    Assumes variable type blocks ends with '_x', '_xx', '_xxx', '_vxx',
    or "_xx_ts"
    """

    regex = "^" + block_name + r"\w*$"
    variable_suffixes = ["_x", "_xx", "_xxx", "_vxx", "_xx_ts"]
    for suffix in variable_suffixes:
        if block_name.endswith(suffix):
            base_name, _, _ = block_name.rpartition(suffix)
            var_suffix = suffix.replace("x", r"\w")
            regex = "^" + base_name + var_suffix + r"\w*$"
            break
    return regex


def docstring_from_make(key, imports, make):
    """
    Extract the documentation from the python __doc__ strings
    By importing it and checking a truncated make

    Args:
        key: the block key
        imports: a list of import statements (string) to execute
        make: block constructor template

    Returns:
        a list of tuples (block_name, doc string)
    """

    try:
        blk_cls = make.partition('(')[0].strip()
        if '$' in blk_cls:
            raise ValueError('Not an identifier')
        ns = dict()
        exec(imports.strip(), ns)
        blk = eval(blk_cls, ns)
        doc_strings = {key: blk.__doc__}

    except (ImportError, AttributeError, SyntaxError, ValueError):
        doc_strings = docstring_guess_from_key(key)

    return doc_strings


###############################################################################
# Manage docstring extraction in separate process
###############################################################################

class SubprocessLoader(object):
    """
    Start and manage docstring extraction process
    Manages subprocess and handles RPC.
    """

    BOOTSTRAP = "import runpy; runpy.run_path({!r}, run_name='__worker__')"
    AUTH_CODE = random.random()  # sort out unwanted output of worker process
    RESTART = 5  # number of worker restarts before giving up
    DONE = object()  # sentinel value to signal end-of-queue

    def __init__(self, callback_query_result, callback_finished=None):
        self.callback_query_result = callback_query_result
        self.callback_finished = callback_finished or (lambda: None)

        self._queue = queue.Queue()
        self._thread = None
        self._worker = None
        self._shutdown = threading.Event()
        self._last_cmd = None

    def start(self):
        """ Start the worker process handler thread """
        if self._thread is not None:
            return
        self._shutdown.clear()
        thread = self._thread = threading.Thread(target=self.run_worker)
        thread.daemon = True
        thread.start()

    def run_worker(self):
        """ Read docstring back from worker stdout and execute callback. """
        for _ in range(self.RESTART):
            if self._shutdown.is_set():
                break
            try:
                self._worker = subprocess.Popen(
                    args=(sys.executable, '-uc',
                          self.BOOTSTRAP.format(__file__)),
                    stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE
                )
                self._handle_worker()

            except (OSError, IOError):
                msg = "Warning: restarting the docstring loader"
                cmd, args = self._last_cmd
                if cmd == 'query':
                    msg += " (crashed while loading {0!r})".format(args[0])
                print(msg, file=sys.stderr)
                continue  # restart
            else:
                break  # normal termination, return
            finally:
                if self._worker:
                    self._worker.stdin.close()
                    self._worker.stdout.close()
                    self._worker.stderr.close()
                    self._worker.terminate()
                    self._worker.wait()
        else:
            print("Warning: docstring loader crashed too often", file=sys.stderr)
        self._thread = None
        self._worker = None
        self.callback_finished()

    def _handle_worker(self):
        """ Send commands and responses back from worker. """
        assert '1' == self._worker.stdout.read(1).decode('utf-8')
        for cmd, args in iter(self._queue.get, self.DONE):
            self._last_cmd = cmd, args
            self._send(cmd, args)
            cmd, args = self._receive()
            self._handle_response(cmd, args)

    def _send(self, cmd, args):
        """ Send a command to worker's stdin """
        fd = self._worker.stdin
        query = json.dumps((self.AUTH_CODE, cmd, args))
        fd.write(query.encode('utf-8'))
        fd.write(b'\n')
        fd.flush()

    def _receive(self):
        """ Receive response from worker's stdout """
        for line in iter(self._worker.stdout.readline, ''):
            try:
                key, cmd, args = json.loads(line.decode('utf-8'))
                if key != self.AUTH_CODE:
                    raise ValueError('Got wrong auth code')
                return cmd, args
            except ValueError:
                if self._worker.poll():
                    raise IOError("Worker died")
                else:
                    continue  # ignore invalid output from worker
        else:
            raise IOError("Can't read worker response")

    def _handle_response(self, cmd, args):
        """ Handle response from worker, call the callback """
        if cmd == 'result':
            key, docs = args
            self.callback_query_result(key, docs)
        elif cmd == 'error':
            print(args)
        else:
            print("Unknown response:", cmd, args, file=sys.stderr)

    def query(self, key, imports=None, make=None):
        """ Request docstring extraction for a certain key """
        if self._thread is None:
            self.start()
        if imports and make:
            self._queue.put(('query', (key, imports, make)))
        else:
            self._queue.put(('query_key_only', (key,)))

    def finish(self):
        """ Signal end of requests """
        self._queue.put(self.DONE)

    def wait(self):
        """ Wait for the handler thread to die """
        if self._thread:
            self._thread.join()

    def terminate(self):
        """ Terminate the worker and wait """
        self._shutdown.set()
        try:
            self._worker.terminate()
            self.wait()
        except (OSError, AttributeError):
            pass


###############################################################################
# Main worker entry point
###############################################################################

def worker_main():
    """
    Main entry point for the docstring extraction process.
    Manages RPC with main process through stdin/stdout.
    Runs a docstring extraction for each key it read on stdin.
    """
    def send(code, cmd, args):
        json.dump((code, cmd, args), sys.stdout)
        sys.stdout.write('\n')
        # fluh out to get new commands from the queue into stdin
        sys.stdout.flush()

    sys.stdout.write('1')
    # flush out to signal the main process we are ready for new commands
    sys.stdout.flush()
    for line in iter(sys.stdin.readline, ''):
        code, cmd, args = json.loads(line)
        try:
            if cmd == 'query':
                key, imports, make = args
                send(code, 'result', (key, docstring_from_make(key, imports, make)))
            elif cmd == 'query_key_only':
                key, = args
                send(code, 'result', (key, docstring_guess_from_key(key)))
            elif cmd == 'exit':
                break
        except Exception as e:
            send(code, 'error', repr(e))


if __name__ == '__worker__':
    worker_main()

elif __name__ == '__main__':
    def callback(key, docs):
        print(key)
        for match, doc in docs.items():
            print('-->', match)
            print(str(doc).strip())
            print()
        print()

    r = SubprocessLoader(callback)

    # r.query('analog_feedforward_agc_cc')
    # r.query('uhd_source')
    r.query('expr_utils_graph')
    r.query('blocks_add_cc')
    r.query('blocks_add_cc', ['import gnuradio.blocks'],
            'gnuradio.blocks.add_cc(')
    # r.query('analog_feedforward_agc_cc')
    # r.query('uhd_source')
    # r.query('uhd_source')
    # r.query('analog_feedforward_agc_cc')
    r.finish()
    # r.terminate()
    r.wait()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: MIT
#

from gnuradio import gr, gr_unittest, blocks
import pmt
import numpy as np


class qa_burst_to_stream(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_instance(self):
        instance = blocks.burst_to_stream(gr.sizeof_char, 'packet_len')

    def test_one_burst(self):
        burst2stream = blocks.burst_to_stream(
            gr.sizeof_float, 'packet_len', True)
        packet_len = 100
        packet = [1.0] * packet_len
        packet_len_tag = gr.tag_utils.python_to_tag((
            0, pmt.intern('packet_len'),
            pmt.from_long(packet_len), pmt.intern('src')))
        packet_source = blocks.vector_source_f(
            packet, False, 1, [packet_len_tag])
        # We collect many samples to have a chance of seeing our packet
        nsamples_collect = 2**20
        head = blocks.head(gr.sizeof_float, nsamples_collect)
        sink = blocks.vector_sink_f(1, nsamples_collect)
        self.tb.connect(packet_source, burst2stream, head, sink)

        self.tb.run()

        data = np.array(sink.data(), dtype='float32')
        self.assertEqual(set(data), {0.0, 1.0})
        packet_start = np.where(data == 1.0)[0][0]
        np.testing.assert_equal(data[packet_start:][:+packet_len], packet)

        tags = sink.tags()
        self.assertEqual(len(tags), 1)  # only one packet_len tag
        tag = tags[0]
        self.assertEqual(pmt.symbol_to_string(tag.key), 'packet_len')
        self.assertEqual(pmt.to_long(tag.value), packet_len)
        self.assertEqual(packet_start, tag.offset)


if __name__ == '__main__':
    gr_unittest.run(qa_burst_to_stream)



#!/usr/bin/env python
#
# Copyright 2014,2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

"""
TXs a waveform (either from a file, or a sinusoid) in a frequency-hopping manner.
"""

import argparse
import numpy
import pmt
from gnuradio import gr
from gnuradio import blocks
from gnuradio import uhd


def setup_parser():
    """ Setup the parser for the frequency hopper. """
    parser = argparse.ArgumentParser(
        description="Transmit a signal in a frequency-hopping manner, using tx_freq tags."
    )
    parser.add_argument(
        '-i', '--input-file',
        help="File with samples to transmit. If left out, will transmit a sinusoid.")
    parser.add_argument(
        "-a", "--args", default="",
        help="UHD device address args.")
    parser.add_argument(
        "--spec", default="",
        help="UHD subdev spec.")
    parser.add_argument(
        "--antenna", default="",
        help="UHD antenna settings.")
    parser.add_argument(
        "--gain", default=None, type=float,
        help="USRP gain (defaults to mid-point in dB).")
    parser.add_argument(
        "-r", "--rate", type=float, default=1e6,
        help="Sampling rate")
    parser.add_argument(
        "-N", "--samp-per-burst", type=int, default=10000,
        help="Samples per burst")
    parser.add_argument(
        "-t", "--hop-time", type=float, default=1000,
        help="Time between hops in milliseconds. This must be larger than or "
             "equal to the burst duration as set by --samp-per-burst")
    parser.add_argument(
        "-f", "--freq", type=float, default=2.45e9,
        help="Base frequency. This is the middle channel frequency at which "
             "the USRP will Tx.")
    parser.add_argument(
        "--dsp", action='store_true',
        help="DSP tuning only.")
    parser.add_argument(
        "-d", "--freq-delta", type=float, default=1e6,
        help="Channel spacing.")
    parser.add_argument(
        "-c", "--num-channels", type=int, default=5,
        help="Number of channels.")
    parser.add_argument(
        "-B", "--num-bursts", type=int, default=30,
        help="Number of bursts to transmit before terminating.")
    parser.add_argument(
        "-p", "--post-tuning", action='count',
        help="Tune after transmitting. Default is to tune immediately before "
             "transmitting.")
    parser.add_argument(
        "-v", "--verbose", action='count',
        help="Print more information. The morer the printier.")
    return parser


class FrequencyHopperSrc(gr.hier_block2):
    """ Provides tags for frequency hopping """

    def __init__(
        self,
        n_bursts, n_channels,
        freq_delta, base_freq, dsp_tuning,
        burst_length, base_time, hop_time,
        post_tuning=False,
        tx_gain=0,
        verbose=False
    ):
        gr.hier_block2.__init__(
            self, "FrequencyHopperSrc",
            gr.io_signature(1, 1, gr.sizeof_gr_complex),
            gr.io_signature(1, 1, gr.sizeof_gr_complex),
        )
        n_samples_total = n_bursts * burst_length
        lowest_frequency = base_freq - numpy.floor(n_channels / 2) * freq_delta
        self.hop_sequence = [lowest_frequency + n *
                             freq_delta for n in range(n_channels)]
        numpy.random.shuffle(self.hop_sequence)
        # Repeat that:
        self.hop_sequence = [self.hop_sequence[x % n_channels]
                             for x in range(n_bursts)]
        if verbose:
            print("Hop Frequencies  | Hop Pattern")
            print("=================|================================")
            for f in self.hop_sequence:
                print("{:6.3f} MHz      |  ".format(f / 1e6), end='')
                if n_channels < 50:
                    print(" " * int((f - base_freq) / freq_delta) + "#")
                else:
                    print("\n")
            print("=================|================================")
        # There's no real point in setting the gain via tag for this application,
        # but this is an example to show you how to do it.
        gain_tag = gr.tag_t()
        gain_tag.offset = 0
        gain_tag.key = pmt.string_to_symbol('tx_command')
        gain_tag.value = pmt.to_pmt({'gain': tx_gain})
        tag_list = [gain_tag, ]
        for i in range(len(self.hop_sequence)):
            time = pmt.cons(
                pmt.from_uint64(int(base_time + i * hop_time + 0.01)),
                pmt.from_double((base_time + i * hop_time + 0.01) % 1),
            )
            tune_tag = gr.tag_t()
            tune_tag.offset = i * burst_length
            # TODO dsp_tuning should also be able to do post_tuning
            if i > 0 and post_tuning and not dsp_tuning:
                tune_tag.offset -= 1  # Move it to last sample of previous burst
            if dsp_tuning:
                tune_tag.key = pmt.string_to_symbol('tx_command')
                tune_tag.value = pmt.to_pmt(
                    {'lo_freq': base_freq, 'dsp_freq': base_freq - self.hop_sequence[i]})
                tune_tag.value = pmt.dict_add(
                    tune_tag.value, pmt.intern("time"), time)
            else:
                tune_tag.key = pmt.string_to_symbol('tx_command')
                tune_tag.value = pmt.to_pmt({'freq': self.hop_sequence[i]})
                tune_tag.value = pmt.dict_add(
                    tune_tag.value, pmt.intern('time'), time)
            tag_list.append(tune_tag)
            length_tag = gr.tag_t()
            length_tag.offset = i * burst_length
            length_tag.key = pmt.string_to_symbol('packet_len')
            length_tag.value = pmt.from_long(burst_length)
            tag_list.append(length_tag)
            time_tag = gr.tag_t()
            time_tag.offset = i * burst_length
            time_tag.key = pmt.string_to_symbol('tx_time')
            time_tag.value = pmt.make_tuple(
                pmt.car(time),
                pmt.cdr(time)
            )
            tag_list.append(time_tag)
        tag_source = blocks.vector_source_c(
            (1.0,) * n_samples_total, repeat=False, tags=tag_list)
        mult = blocks.multiply_cc()
        self.connect(self, mult, self)
        self.connect(tag_source, (mult, 1))


class FlowGraph(gr.top_block):
    """ Flow graph that does the frequency hopping. """

    def __init__(self, args):
        gr.top_block.__init__(self)
        if args.input_file is not None:
            src = blocks.file_source(
                gr.sizeof_gr_complex, args.input_file, repeat=True)
        else:
            src = blocks.vector_source_c((.5,) * int(1e6) * 2, repeat=True)
        # Setup USRP
        self.usrp = uhd.usrp_sink(
            args.args,
            uhd.stream_args('fc32'),
            "packet_len"
        )
        if args.spec:
            self.usrp.set_subdev_spec(args.spec, 0)
        if args.antenna:
            self.usrp.set_antenna(args.antenna, 0)
        self.usrp.set_samp_rate(args.rate)
        # Gain is set in the hopper block
        if not args.gain:
            gain_range = self.usrp.get_gain_range()
            args.gain = float(gain_range.start() + gain_range.stop()) / 2.0
        print("-- Setting gain to {} dB".format(args.gain))
        if not self.usrp.set_center_freq(args.freq):
            print('[ERROR] Failed to set base frequency.')
            exit(1)
        hopper_block = FrequencyHopperSrc(
            args.num_bursts, args.num_channels,
            args.freq_delta, args.freq, args.dsp,
            args.samp_per_burst, 1.0, args.hop_time / 1000.,
            args.post_tuning,
            args.gain,
            args.verbose,
        )
        self.connect(src, hopper_block, self.usrp)


def print_hopper_stats(args):
    """ Nothing to do with Grace Hopper """
    print("""
Parameter          | Value
===================+=========================
Hop Interval       | {hop_time} ms
Burst duration     | {hop_duration} ms
Lowest Frequency   | {lowest_freq:6.3f} MHz
Highest Frequency  | {highest_freq:6.3f} MHz
Frequency spacing  | {freq_delta:6.4f} MHz
Number of channels | {num_channels}
Sampling rate      | {rate} Msps
Transmit Gain      | {gain} dB
===================+=========================
    """.format(
        hop_time=args.hop_time,
        hop_duration=1000.0 / args.rate * args.samp_per_burst,
        gain=args.gain if args.gain else "(midpoint)",
        lowest_freq=args.freq / 1e6,
        highest_freq=(args.freq + (args.num_channels - 1) *
                      args.freq_delta) / 1e6,
        freq_delta=args.freq_delta / 1e6,
        num_channels=args.num_channels,
        rate=args.rate / 1e6,
    ))


def main():
    """ Go, go, go! """
    args = setup_parser().parse_args()
    if (1.0 * args.samp_per_burst / args.rate) > args.hop_time * 1e-3:
        print("Burst duration must be smaller than hop time.")
        exit(1)
    if args.verbose:
        print_hopper_stats(args)
    top_block = FlowGraph(args)
    print("Starting to hop, skip and jump... press Ctrl+C to exit.")
    top_block.usrp.set_time_now(uhd.time_spec(0.0))
    top_block.run()


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr
import numpy as np
from scipy.stats import norm, laplace, rayleigh
from matplotlib import pyplot as plt

# NOTE: scipy and matplotlib are optional packages and not included in the default gnuradio dependencies

#*** SETUP ***#

# Number of realisations per histogram
num_tests = 1000000

# Set number of bins in histograms
uniform_num_bins = 31
gauss_num_bins = 31
rayleigh_num_bins = 31
laplace_num_bins = 31

rndm = gr.random()  # instance of gnuradio random class (gr::random)

print('All histograms contain', num_tests, 'realisations.')

#*** GENERATE DATA ***#

uniform_values = np.zeros(num_tests)
gauss_values = np.zeros(num_tests)
rayleigh_values = np.zeros(num_tests)
laplace_values = np.zeros(num_tests)

for k in range(num_tests):
    uniform_values[k] = rndm.ran1()
    gauss_values[k] = rndm.gasdev()
    rayleigh_values[k] = rndm.rayleigh()
    laplace_values[k] = rndm.laplacian()

#*** HISTOGRAM DATA AND CALCULATE EXPECTED COUNTS ***#

uniform_bins = np.linspace(0, 1, uniform_num_bins)
gauss_bins = np.linspace(-8, 8, gauss_num_bins)
laplace_bins = np.linspace(-8, 8, laplace_num_bins)
rayleigh_bins = np.linspace(0, 10, rayleigh_num_bins)

uniform_hist = np.histogram(uniform_values, uniform_bins)
gauss_hist = np.histogram(gauss_values, gauss_bins)
rayleigh_hist = np.histogram(rayleigh_values, rayleigh_bins)
laplace_hist = np.histogram(laplace_values, laplace_bins)

uniform_expected = np.zeros(uniform_num_bins - 1)
gauss_expected = np.zeros(gauss_num_bins - 1)
rayleigh_expected = np.zeros(rayleigh_num_bins - 1)
laplace_expected = np.zeros(laplace_num_bins - 1)

for k in range(len(uniform_hist[0])):
    uniform_expected[k] = num_tests / float(uniform_num_bins - 1)

for k in range(len(gauss_hist[0])):
    gauss_expected[k] = float(
        norm.cdf(gauss_hist[1][k + 1]) - norm.cdf(gauss_hist[1][k])) * num_tests

for k in range(len(rayleigh_hist[0])):
    rayleigh_expected[k] = float(rayleigh.cdf(
        rayleigh_hist[1][k + 1]) - rayleigh.cdf(rayleigh_hist[1][k])) * num_tests

for k in range(len(laplace_hist[0])):
    laplace_expected[k] = float(laplace.cdf(
        laplace_hist[1][k + 1]) - laplace.cdf(laplace_hist[1][k])) * num_tests

#*** PLOT HISTOGRAMS AND EXPECTATIONS TAKEN FROM SCIPY ***#

uniform_bins_center = uniform_bins[0:-1] + \
    (uniform_bins[1] - uniform_bins[0]) / 2.0
gauss_bins_center = gauss_bins[0:-1] + (gauss_bins[1] - gauss_bins[0]) / 2.0
rayleigh_bins_center = rayleigh_bins[0:-1] + \
    (rayleigh_bins[1] - rayleigh_bins[0]) / 2.0
laplace_bins_center = laplace_bins[0:-1] + \
    (laplace_bins[1] - laplace_bins[0]) / 2.0

plt.figure(1)

plt.subplot(2, 1, 1)
plt.plot(uniform_bins_center,
         uniform_hist[0], 's--', uniform_bins_center, uniform_expected, 'o:')
plt.xlabel('Bins'), plt.ylabel('Count'), plt.title('Uniform: Distribution')
plt.legend(['histogram gr::random', 'calculation scipy'], loc=1)

plt.subplot(2, 1, 2)
plt.plot(uniform_bins_center, uniform_hist[0] / uniform_expected, 'rs--')
plt.xlabel('Bins'), plt.ylabel('Relative deviation'), plt.title(
    'Uniform: Relative deviation to scipy')

plt.figure(2)

plt.subplot(2, 1, 1)
plt.plot(gauss_bins_center, gauss_hist[0], 's--',
         gauss_bins_center, gauss_expected, 'o:')
plt.xlabel('Bins'), plt.ylabel('Count'), plt.title('Gauss: Distribution')
plt.legend(['histogram gr::random', 'calculation scipy'], loc=1)

plt.subplot(2, 1, 2)
plt.plot(gauss_bins_center, gauss_hist[0] / gauss_expected, 'rs--')
plt.xlabel('Bins'), plt.ylabel('Relative deviation'), plt.title(
    'Gauss: Relative deviation to scipy')

plt.figure(3)

plt.subplot(2, 1, 1)
plt.plot(rayleigh_bins_center,
         rayleigh_hist[0], 's--', rayleigh_bins_center, rayleigh_expected, 'o:')
plt.xlabel('Bins'), plt.ylabel('Count'), plt.title('Rayleigh: Distribution')
plt.legend(['histogram gr::random', 'calculation scipy'], loc=1)


plt.subplot(2, 1, 2)
plt.plot(rayleigh_bins_center, rayleigh_hist[0] / rayleigh_expected, 'rs--')
plt.xlabel('Bins'), plt.ylabel('Relative deviation'), plt.title(
    'Rayleigh: Relative deviation to scipy')

plt.figure(4)

plt.subplot(2, 1, 1)
plt.plot(laplace_bins_center,
         laplace_hist[0], 's--', laplace_bins_center, laplace_expected, 'o:')
plt.xlabel('Bins'), plt.ylabel('Count'), plt.title('Laplace: Distribution')
plt.legend(['histogram gr::random', 'calculation scipy'], loc=1)

plt.subplot(2, 1, 2)
plt.plot(laplace_bins_center, laplace_hist[0] / laplace_expected, 'rs--')
plt.xlabel('Bins'), plt.ylabel('Relative deviation'), plt.title(
    'Laplace: Relative deviation to scipy')

plt.show()



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
# Copyright 2023 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

# This is similar to _qa_helper.py but uses async_encoder and async_decoder
# instead of extended_encoder and extended_decoder.


import numpy as np

from gnuradio import gr, blocks, fec, pdu
# Must use absolute import here because this file is not installed as part
# of the module
from gnuradio.fec import async_encoder, async_decoder
import pmt

from _qa_helper import map_bb


class _qa_helper_async(gr.top_block):

    def __init__(self, frame_size, enc, dec, packed, rev_pack):
        gr.top_block.__init__(self, "_qa_helper_async")

        self.enc = enc
        self.dec = dec
        self.frame_size = frame_size
        # These options are for the decoder only. The encoder is always run in
        # unpacked mode.
        self.packed = packed
        self.rev_pack = rev_pack

        self.async_encoder = async_encoder(enc)
        self.async_decoder = async_decoder(
            dec, packed=packed, rev_pack=rev_pack)

        num_frames = 64
        frame_size_bits = 8 * frame_size
        data_in = np.random.randint(
            2, size=frame_size_bits * num_frames, dtype='uint8')
        tags = [
            gr.tag_utils.python_to_tag((
                frame_size_bits * j, pmt.intern('packet_len'),
                pmt.from_long(frame_size_bits), pmt.intern('src')))
            for j in range(num_frames)]
        self.src = blocks.vector_source_b(data_in, False, tags=tags)
        self.tagged_to_pdu_enc = pdu.tagged_stream_to_pdu(
            gr.types.byte_t, 'packet_len')
        self.pdu_to_tagged_enc = pdu.pdu_to_tagged_stream(
            gr.types.byte_t, 'packet_len')
        self.map = map_bb([-1, 1])
        self.to_float = blocks.char_to_float(1)
        self.tagged_to_pdu_dec = pdu.tagged_stream_to_pdu(
            gr.types.float_t, 'packet_len')
        self.pdu_to_tagged_dec = pdu.pdu_to_tagged_stream(
            gr.types.byte_t, 'packet_len')
        self.snk_input = blocks.vector_sink_b()
        self.snk_output = blocks.vector_sink_b()
        if packed:
            self.unpack_decoder = blocks.packed_to_unpacked_bb(
                1, gr.GR_LSB_FIRST if rev_pack else gr.GR_MSB_FIRST)

        self.connect(self.src, self.tagged_to_pdu_enc)
        self.msg_connect((self.tagged_to_pdu_enc, 'pdus'),
                         (self.async_encoder, 'in'))
        self.msg_connect((self.async_encoder, 'out'),
                         (self.pdu_to_tagged_enc, 'pdus'))
        self.connect(
            self.pdu_to_tagged_enc, self.map, self.to_float,
            self.tagged_to_pdu_dec)
        self.msg_connect((self.tagged_to_pdu_dec, 'pdus'),
                         (self.async_decoder, 'in'))
        self.msg_connect((self.async_decoder, 'out'),
                         (self.pdu_to_tagged_dec, 'pdus'))
        self.connect(self.src, self.snk_input)
        if packed:
            self.connect(
                self.pdu_to_tagged_dec, self.unpack_decoder, self.snk_output)
        else:
            self.connect(self.pdu_to_tagged_dec, self.snk_output)


if __name__ == '__main__':
    frame_size = 30
    enc = fec.dummy_encoder_make(frame_size * 8)
    dec = fec.dummy_decoder.make(frame_size * 8)

    for packed in [True, False]:
        for rev_pack in [True, False]:
            tb = _qa_helper_async(frame_size, enc, dec, packed, rev_pack)
            tb.run()

            errs = 0
            for i, o in zip(tb.snk_input.data(), tb.snk_output.data()):
                if i - o != 0:
                    errs += 1

            if errs == 0:
                print("Decoded properly")
            else:
                print("Problem Decoding")



#!/usr/bin/env python
#
# Copyright 2011-2013,2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" Test digital.mpsk_snr_est_cc """

import random

from gnuradio import gr, gr_unittest, digital, blocks


def random_bit():
    """Create random bits using random() rather than randint(). The latter
    changed for Python 3.2."""
    return random.random() > .5


def get_cplx():
    "Return a BPSK symbol (complex)"
    return complex(2 * random_bit() - 1, 0)


def get_n_cplx():
    "Return random, normal-distributed complex number"
    return complex(random.random() - 0.5, random.random() - 0.5)


class test_mpsk_snr_est(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()
        random.seed(0)  # make repeatable
        N = 10000
        self._noise = [get_n_cplx() for _ in range(N)]
        self._bits = [get_cplx() for _ in range(N)]

    def tearDown(self):
        self.tb = None

    def mpsk_snr_est_setup(self, op):
        result = []
        for i in range(1, 6):
            src_data = [b + (i * n) for b, n in zip(self._bits, self._noise)]
            src = blocks.vector_source_c(src_data)
            dst = blocks.null_sink(gr.sizeof_gr_complex)
            tb = gr.top_block()
            tb.connect(src, op)
            tb.connect(op, dst)
            tb.run()  # run the graph and wait for it to finish

            result.append(op.snr())
        return result

    def test_mpsk_snr_est_simple(self):
        expected_result = [8.20, 4.99, 3.23, 2.01, 1.03]

        N = 10000
        alpha = 0.001
        op = digital.mpsk_snr_est_cc(digital.SNR_EST_SIMPLE, N, alpha)

        actual_result = self.mpsk_snr_est_setup(op)
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 2)

    def test_mpsk_snr_est_skew(self):
        expected_result = [8.31, 1.83, -1.68, -3.56, -4.68]

        N = 10000
        alpha = 0.001
        op = digital.mpsk_snr_est_cc(digital.SNR_EST_SKEW, N, alpha)

        actual_result = self.mpsk_snr_est_setup(op)
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 2)

    def test_mpsk_snr_est_m2m4(self):
        expected_result = [8.01, 3.19, 1.97, 2.15, 2.65]

        N = 10000
        alpha = 0.001
        op = digital.mpsk_snr_est_cc(digital.SNR_EST_M2M4, N, alpha)

        actual_result = self.mpsk_snr_est_setup(op)
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 2)

    def test_mpsk_snr_est_svn(self):
        expected_result = [7.91, 3.01, 1.77, 1.97, 2.49]

        N = 10000
        alpha = 0.001
        op = digital.mpsk_snr_est_cc(digital.SNR_EST_SVR, N, alpha)

        actual_result = self.mpsk_snr_est_setup(op)
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 2)

    def test_probe_mpsk_snr_est_m2m4(self):
        expected_result = [8.01, 3.19, 1.97, 2.15, 2.65]

        actual_result = []
        for i in range(1, 6):
            src_data = [b + (i * n) for b, n in zip(self._bits, self._noise)]

            src = blocks.vector_source_c(src_data)

            N = 10000
            alpha = 0.001
            op = digital.probe_mpsk_snr_est_c(digital.SNR_EST_M2M4, N, alpha)

            tb = gr.top_block()
            tb.connect(src, op)
            tb.run()               # run the graph and wait for it to finish

            actual_result.append(op.snr())
        self.assertFloatTuplesAlmostEqual(expected_result, actual_result, 2)


if __name__ == '__main__':
    # Test various SNR estimators; we're not using a Gaussian
    # noise source, so these estimates have no real meaning;
    # just a sanity check.
    gr_unittest.run(test_mpsk_snr_est)



#!/usr/bin/env python
#
# Copyright 2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, gr_unittest, filter, blocks


def sig_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.sin(2. * math.pi * freq * x) for x in t]
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


def const_source_f(amp, N):
    y = N * [amp, ]
    return y


class test_mmse_resampler(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_ff(self):
        N = 10000        # number of samples to use
        fs = 1000        # baseband sampling rate
        rrate = 1.123    # resampling rate

        freq = 10
        data = sig_source_f(fs, freq, 1, N)
        signal = blocks.vector_source_f(data)
        op = filter.mmse_resampler_ff(0, rrate)
        snk = blocks.vector_sink_f()

        self.tb.connect(signal, op, snk)
        self.tb.run()

        Ntest = 5000
        L = len(snk.data())
        t = [float(x) / (fs / rrate) for x in range(L)]

        phase = 0.1884
        expected_data = [math.sin(2. * math.pi * freq * x + phase) for x in t]

        dst_data = snk.data()

        self.assertFloatTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 3)

    def test_002_cc(self):
        N = 10000        # number of samples to use
        fs = 1000        # baseband sampling rate
        rrate = 1.123    # resampling rate

        freq = 10
        data = sig_source_c(fs, freq, 1, N)
        signal = blocks.vector_source_c(data)
        op = filter.mmse_resampler_cc(0.0, rrate)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, op, snk)
        self.tb.run()

        Ntest = 5000
        L = len(snk.data())
        t = [float(x) / (fs / rrate) for x in range(L)]

        phase = 0.1884
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 3)

    def test_003_ff(self):
        N = 10000        # number of samples to use
        fs = 1000        # baseband sampling rate
        rrate = 1.123    # resampling rate

        freq = 10
        data = sig_source_f(fs, freq, 1, N)
        ctrl = const_source_f(rrate, N)
        signal = blocks.vector_source_f(data)
        control = blocks.vector_source_f(ctrl)
        op = filter.mmse_resampler_ff(0, 1)
        snk = blocks.vector_sink_f()

        self.tb.connect(signal, op, snk)
        self.tb.connect(control, (op, 1))
        self.tb.run()

        Ntest = 5000
        L = len(snk.data())
        t = [float(x) / (fs / rrate) for x in range(L)]

        phase = 0.1884
        expected_data = [math.sin(2. * math.pi * freq * x + phase) for x in t]

        dst_data = snk.data()

        self.assertFloatTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 3)

    def test_004_cc(self):
        N = 10000        # number of samples to use
        fs = 1000        # baseband sampling rate
        rrate = 1.123    # resampling rate

        freq = 10
        data = sig_source_c(fs, freq, 1, N)
        ctrl = const_source_f(rrate, N)
        signal = blocks.vector_source_c(data)
        control = blocks.vector_source_f(ctrl)
        op = filter.mmse_resampler_cc(0.0, 1)
        snk = blocks.vector_sink_c()

        self.tb.connect(signal, op, snk)
        self.tb.connect(control, (op, 1))
        self.tb.run()

        Ntest = 5000
        L = len(snk.data())
        t = [float(x) / (fs / rrate) for x in range(L)]

        phase = 0.1884
        expected_data = [
            math.cos(
                2. *
                math.pi *
                freq *
                x +
                phase) +
            1j *
            math.sin(
                2. *
                math.pi *
                freq *
                x +
                phase) for x in t]

        dst_data = snk.data()

        self.assertComplexTuplesAlmostEqual(
            expected_data[-Ntest:], dst_data[-Ntest:], 3)


if __name__ == '__main__':
    gr_unittest.run(test_mmse_resampler)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
from gnuradio import fec
import numpy as np


class test_ccsds_27 (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_ccsds_27(self):
        src_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        expected = [0, 0, 0, 0, 1, 2, 3, 4, 5, 6]
        src = blocks.vector_source_b(src_data)
        enc = fec.encode_ccsds_27_bb()
        b2f = blocks.char_to_float()
        add = blocks.add_const_ff(-0.5)
        mul = blocks.multiply_const_ff(2.0)
        dec = fec.decode_ccsds_27_fb()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, enc, b2f, add, mul, dec, dst)
        self.tb.run()
        dst_data = dst.data()
        self.assertEqual(expected, dst_data)

    def test_ccsds_27_long(self):
        src_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10] * 100000
        src = blocks.vector_source_b(src_data)
        enc = fec.encode_ccsds_27_bb()
        b2f = blocks.char_to_float()
        add = blocks.add_const_ff(-0.5)
        mul = blocks.multiply_const_ff(2.0)
        dec = fec.decode_ccsds_27_fb()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, enc, b2f, add, mul, dec, dst)
        self.tb.run()
        dst_data = dst.data()
        self.assertEqual([0, 0, 0, 0] + src_data[:-4], dst_data)


if __name__ == '__main__':
    gr_unittest.run(test_ccsds_27)



#!/usr/bin/env python
#
# Copyright 2006,2007,2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import weakref

from gnuradio import blocks, gr, gr_unittest
import pmt


class block_with_message_output(gr.basic_block):

    def __init__(self):
        gr.basic_block.__init__(self,
                                "block_with_message_output",
                                in_sig=None,
                                out_sig=None)
        self.message_port_register_out(pmt.intern("test"))


class block_with_message_input(gr.basic_block):

    def __init__(self):
        gr.basic_block.__init__(self,
                                "block_with_message_input",
                                in_sig=None,
                                out_sig=None)
        self.message_port_register_in(pmt.intern("test"))


class hier_block_with_message_output(gr.hier_block2):

    def __init__(self):
        gr.hier_block2.__init__(self,
                                "hier_block_with_message_output",
                                gr.io_signature(0, 0, 0),  # Input signature
                                gr.io_signature(0, 0, 0))  # Output signature
        self.message_port_register_hier_out("test")
        self.block = block_with_message_output()
        self.msg_connect(self.block, "test", self, "test")


class hier_block_with_message_input(gr.hier_block2):

    def __init__(self):
        gr.hier_block2.__init__(self,
                                "hier_block_with_message_output",
                                gr.io_signature(0, 0, 0),  # Input signature
                                gr.io_signature(0, 0, 0))  # Output signature
        self.message_port_register_hier_in("test")
        self.block = block_with_message_input()
        self.msg_connect(self, "test", self.block, "test")


class hier_block_with_message_inout(gr.hier_block2):

    def __init__(self):
        gr.hier_block2.__init__(self,
                                "hier_block_with_message_inout",
                                gr.io_signature(0, 0, 0),  # Input signature
                                gr.io_signature(0, 0, 0))  # Output signature
        self.message_port_register_hier_in("test")
        self.message_port_register_hier_out("test")
        self.input = block_with_message_input()
        self.msg_connect(self, "test", self.input, "test")
        self.output = block_with_message_output()
        self.msg_connect(self.output, "test", weakref.proxy(self), "test")


class test_hier_block2_message_connections(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def run_top_block(self):
        self.tb.start()
        self.tb.stop()
        self.tb.wait()

    def assert_has_subscription(self, sender, send_port, receiver,
                                receive_port):
        """assert that the given sender block has a subscription for the given
        receiver block on the appropriate send and receive ports

        :param sender: a block sptr to the message sender
        :param string send_port: the port messages are being sent on
        :param receiver: a block sptr to the message receiver
        :param string receive_port: the port messages are being received on
        """
        subs = sender.message_subscribers(pmt.intern(send_port))
        self.assertTrue(pmt.list_has(subs, pmt.cons(
            pmt.intern(receiver.to_basic_block().alias()),
            pmt.intern(receive_port))))

    def assert_has_num_subscriptions(self, block, port, number):
        """assert that the given block has the given number of subscriptions
        on the given port

        :param block: a block sptr
        :param string port: the port name
        :param number: the number of subscriptions expected
        """
        subs = block.message_subscribers(pmt.intern(port))
        self.assertEqual(pmt.length(subs), number)

    def test_hier_out_to_normal_in(self):
        message_debug = blocks.message_debug()
        hier = hier_block_with_message_output()

        self.tb.msg_connect(hier, "test", message_debug, "print")
        self.run_top_block()
        self.assert_has_num_subscriptions(hier.block, "test", 1)
        self.assert_has_num_subscriptions(hier, "test", 0)
        self.assert_has_subscription(
            hier.block, "test", message_debug, "print")
        self.tb.msg_disconnect(hier, "test", message_debug, "print")
        self.assert_has_num_subscriptions(hier.block, "test", 0)

    def test_normal_out_to_hier_in(self):
        b = block_with_message_output()
        hier = hier_block_with_message_input()

        self.tb.msg_connect(b, "test", hier, "test")
        self.run_top_block()
        self.assert_has_num_subscriptions(b, "test", 1)
        self.assert_has_subscription(b, "test", hier.block, "test")
        self.tb.msg_disconnect(b, "test", hier, "test")
        self.assert_has_num_subscriptions(b, "test", 0)

    def test_hier_out_to_hier_in(self):
        hier_out = hier_block_with_message_output()
        hier_in = hier_block_with_message_input()

        self.tb.msg_connect(hier_out, "test", hier_in, "test")
        self.run_top_block()
        self.assert_has_num_subscriptions(hier_out, "test", 0)
        self.assert_has_num_subscriptions(hier_out.block, "test", 1)
        self.assert_has_subscription(
            hier_out.block, "test", hier_in.block, "test")
        self.tb.msg_disconnect(hier_out, "test", hier_in, "test")
        self.assert_has_num_subscriptions(hier_out.block, "test", 0)

    def test_normal_in_to_hier_to_normal_out(self):
        hier = hier_block_with_message_inout()
        input = block_with_message_output()
        output = block_with_message_input()

        self.tb.msg_connect(input, "test", hier, "test")
        self.tb.msg_connect(hier, "test", output, "test")
        self.run_top_block()
        self.assert_has_num_subscriptions(input, "test", 1)
        self.assert_has_subscription(input, "test", hier.input, "test")
        self.assert_has_num_subscriptions(hier, "test", 0)
        self.assert_has_num_subscriptions(hier.output, "test", 1)
        self.assert_has_subscription(hier.output, "test", output, "test")
        self.tb.msg_disconnect(input, "test", hier, "test")
        self.tb.msg_disconnect(hier, "test", output, "test")
        self.assert_has_num_subscriptions(input, "test", 0)
        self.assert_has_num_subscriptions(hier.output, "test", 0)

    def test_multiple_connections(self):
        hier = hier_block_with_message_output()
        x = block_with_message_input()
        y = block_with_message_input()

        self.tb.msg_connect(hier, "test", x, "test")
        self.tb.msg_connect(hier, "test", y, "test")
        self.run_top_block()
        self.assert_has_num_subscriptions(hier, "test", 0)
        self.assert_has_num_subscriptions(hier.block, "test", 2)
        self.assert_has_subscription(hier.block, "test", x, "test")
        self.assert_has_subscription(hier.block, "test", y, "test")
        self.tb.msg_disconnect(hier, "test", y, "test")
        self.assert_has_num_subscriptions(hier, "test", 0)
        self.assert_has_num_subscriptions(hier.block, "test", 1)
        self.assert_has_subscription(hier.block, "test", x, "test")
        self.run_top_block()
        self.tb.msg_disconnect(hier, "test", x, "test")
        self.assert_has_num_subscriptions(hier.block, "test", 0)


if __name__ == '__main__':
    gr_unittest.run(test_hier_block2_message_connections)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_stretch(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_stretch_01(self):
        tb = self.tb

        data = 10 * [1, ]
        data0 = [x / 20.0 for x in data]
        data1 = [x / 10.0 for x in data]

        expected_result0 = 10 * [0.05, ]
        expected_result1 = 10 * [0.1, ]

        src0 = blocks.vector_source_f(data0, False)
        src1 = blocks.vector_source_f(data1, False)
        inter = blocks.streams_to_vector(gr.sizeof_float, 2)
        op = blocks.stretch_ff(0.1, 2)
        deinter = blocks.vector_to_streams(gr.sizeof_float, 2)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()

        tb.connect(src0, (inter, 0))
        tb.connect(src1, (inter, 1))
        tb.connect(inter, op)
        tb.connect(op, deinter)
        tb.connect((deinter, 0), dst0)
        tb.connect((deinter, 1), dst1)
        tb.run()

        dst0_data = dst0.data()
        dst1_data = dst1.data()

        self.assertFloatTuplesAlmostEqual(expected_result0, dst0_data, 4)
        self.assertFloatTuplesAlmostEqual(expected_result1, dst1_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_stretch)



import os

from os import path

import pytest

from grc.core.platform import Platform

try:
    os.mkdir(path.join(path.dirname(__file__), 'resources/tests'))
except FileExistsError:
    pass

# Gather blocks
BLOCK_PATHS = []
ROOT = path.join(path.dirname(__file__), '../..')
BLOCK_PATHS = [path.abspath(path.join(ROOT, 'grc/blocks')),
               path.abspath(path.join(ROOT, 'build/gr-uhd/grc'))]
for file_dir in os.scandir(ROOT):
    # If it is a module
    if path.isdir(file_dir) and file_dir.name.startswith("gr-"):
        BLOCK_PATHS.append(path.abspath(path.join(file_dir, "grc")))

# These examples are known to fail and need to be resolved
# But skip now to allow test to pass and prevent further
# regression on the vast majority of examples
BLACKLISTED = ['transmitter_sim_hier.grc',
               'uhd_packet_rx.grc',
               'uhd_packet_tx_tun.grc',
               'packet_loopback_hier.grc',
               'uhd_packet_tx.grc',
               'uhd_packet_rx_tun.grc',
               'formatter_ofdm.grc',
               'ber_curve_gen_ldpc.grc',
               'ber_curve_gen.grc',
               'polar_ber_curve_gen.grc',
               'tpc_ber_curve_gen.grc',
               'comparing_resamplers.grc',
               'pfb_sync_test.grc',
               'soapy_receive.grc',
               'soapy_receive2.grc',
               'soapy_transmit.grc',
               'fm_radio_receiver_soapyremote.grc',
               'fm_radio_receiver_soapy.grc',
               'uhd_siggen_gui.grc',
               'filter_taps_loader.grc',
               'uhd_fft.grc'
               ]


def gather_examples():
    global ROOT, BLACKLISTED
    example_paths = []
    for file_dir in os.scandir(ROOT):
        # If it is a module
        if path.isdir(file_dir) and file_dir.name.startswith("gr-") and not file_dir.name.startswith('gr-trellis'):
            try:
                for pos_ex in os.scandir(path.join(file_dir, "examples")):
                    if path.isfile(pos_ex) and pos_ex.name.endswith(".grc") and not path.basename(pos_ex) in BLACKLISTED:
                        example_paths.append(pos_ex)
                    elif path.isdir(pos_ex):
                        for pos_ex2 in os.scandir(pos_ex):
                            if path.isfile(pos_ex2) and pos_ex2.name.endswith(".grc") and not path.basename(pos_ex2) in BLACKLISTED:
                                example_paths.append(pos_ex2)

            except FileNotFoundError:
                continue
    return example_paths


def print_proper(element):
    if element.is_block:
        return element.name
    return f"{element.parent.name} - {element}"


@pytest.mark.examples
@pytest.mark.parametrize("example", gather_examples())
def test_all_examples(example):
    global BLOCK_PATHS

    print(example.path)

    platform = Platform(
        name='GNU Radio Companion Compiler',
        prefs=None,
        version='0.0.0',
    )
    platform.build_library(BLOCK_PATHS)

    flow_graph = platform.make_flow_graph(example.path)
    flow_graph.rewrite()
    flow_graph.validate()

    assert flow_graph.is_valid(), (example.name, [
        f"{print_proper(elem)}: {msg}" for elem, msg in flow_graph.iter_error_messages()])

    generator = platform.Generator(flow_graph, path.join(
        path.dirname(__file__), 'resources/tests'))
    generator.write()


if __name__ == '__main__':
    examples = gather_examples()
    for example in examples:
        test_all_examples(example)



#!/usr/bin/env python
#
# Copyright 2012-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class qa_plateau_detector_fb (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_t(self):
        #                  | Spur spike 1     | Plateau                  | Spur spike 2
        test_signal = [0, 1, .2, .4, .6, .8, 1, 1, 1, 1, 1, .8, .6, .4, 1, 0]
        expected_sig = [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]
        #                                           | Center of Plateau
        sink = blocks.vector_sink_b()
        self.tb.connect(blocks.vector_source_f(test_signal),
                        blocks.plateau_detector_fb(5), sink)
        self.tb.run()
        self.assertEqual(expected_sig, sink.data())


if __name__ == '__main__':
    gr_unittest.run(qa_plateau_detector_fb)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_map(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def helper(self, symbols):
        src_data = [0, 1, 2, 3, 0, 1, 2, 3]
        expected_data = [symbols[x] for x in src_data]
        src = blocks.vector_source_b(src_data)
        op = digital.map_bb(symbols)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()

        result_data = list(dst.data())
        self.assertEqual(expected_data, result_data)

    def test_001(self):
        symbols = [0, 0, 0, 0]
        self.helper(symbols)

    def test_002(self):
        symbols = [3, 2, 1, 0]
        self.helper(symbols)

    def test_003(self):
        symbols = [8 - 1, 32 - 1, 128, 256 - 1]
        self.helper(symbols)


if __name__ == '__main__':
    gr_unittest.run(test_map)



#!/usr/bin/env python
#
# Copyright 2011, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import unittest
import pmt
import struct


class test_pmt(unittest.TestCase):

    MAXINT32 = (2**31) - 1
    MININT32 = (-MAXINT32) - 1

    def setUp(self):
        from ctypes import sizeof, c_long
        self.sizeof_long = sizeof(c_long)

    def test01(self):
        a = pmt.intern("a")
        b = pmt.from_double(123765)
        d1 = pmt.make_dict()
        d2 = pmt.dict_add(d1, a, b)
        print(d2)

    def test02(self):
        const = 123765
        x_pmt = pmt.from_double(const)
        x_int = pmt.to_double(x_pmt)
        x_float = pmt.to_float(x_pmt)
        self.assertEqual(x_int, const)
        self.assertEqual(x_float, const)

    def test03(self):
        v = pmt.init_f32vector(3, [11, -22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))
        self.assertEqual(pmt.uniform_vector_itemsize(v), 4)

    def test04(self):
        v = pmt.init_f64vector(3, [11, -22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test05(self):
        v = pmt.init_u8vector(3, [11, 22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test06(self):
        v = pmt.init_s8vector(3, [11, -22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test07(self):
        v = pmt.init_u16vector(3, [11, 22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test08(self):
        v = pmt.init_s16vector(3, [11, -22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test09(self):
        v = pmt.init_u32vector(3, [11, 22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test10(self):
        v = pmt.init_s32vector(3, [11, -22, 33])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test11(self):
        v = pmt.init_c32vector(3, [11 + -101j, -22 + 202j, 33 + -303j])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))
        self.assertEqual(pmt.uniform_vector_itemsize(v), 8)

    def test12(self):
        v = pmt.init_c64vector(3, [11 + -101j, -22 + 202j, 33 + -303j])
        s = pmt.serialize_str(v)
        d = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(v, d))

    def test13(self):
        const = self.MAXINT32
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))
        self.assertEqual(const, pmt.to_long(deser))

    def test14(self):
        const = self.MAXINT32 - 1
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))
        self.assertEqual(const, pmt.to_long(deser))

    def test15(self):
        if (self.sizeof_long <= 4):
            return
        const = self.MAXINT32 + 1
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))
        x_long = pmt.to_long(deser)
        self.assertEqual(const, x_long)

    def test16(self):
        const = self.MININT32
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))

    def test17(self):
        const = self.MININT32 + 1
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))
        x_long = pmt.to_long(deser)
        self.assertEqual(const, x_long)

    def test18(self):
        if (self.sizeof_long <= 4):
            return
        const = self.MININT32 - 1
        x_pmt = pmt.from_long(const)
        s = pmt.serialize_str(x_pmt)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.equal(x_pmt, deser))
        x_long = pmt.to_long(deser)
        self.assertEqual(const, x_long)

    def test19(self):
        max_key = pmt.intern("MAX")
        _max = pmt.from_long(self.MAXINT32)
        min_key = pmt.intern("MIN")
        _min = pmt.from_long(self.MININT32)
        d = pmt.make_dict()
        d = pmt.dict_add(d, max_key, _max)
        d = pmt.dict_add(d, min_key, _min)
        s = pmt.serialize_str(d)
        deser = pmt.deserialize_str(s)
        self.assertTrue(pmt.is_dict(deser))
        self.assertTrue(pmt.equal(d, deser))
        p_dict = pmt.to_python(deser)
        self.assertEqual(self.MAXINT32, p_dict["MAX"])
        self.assertEqual(self.MININT32, p_dict["MIN"])

    def test20_absolute_serialization_misc(self):
        # null SERDES
        in_str = b'\x06'
        out_str = pmt.serialize_str(pmt.PMT_NIL)
        self.assertEqual(out_str, in_str)
        in_str = b'\x07\x06\x06'
        out_pmt = pmt.deserialize_str(in_str)
        self.assertTrue(pmt.equal(out_pmt, pmt.PMT_EOF))

        # T/F SERDES
        in_str = b'\x01'
        out_str = pmt.serialize_str(pmt.PMT_F)
        self.assertEqual(out_str, in_str)
        in_str = b'\x00'
        out_pmt = pmt.deserialize_str(in_str)
        self.assertTrue(pmt.equal(out_pmt, pmt.PMT_T))

        # pair SERDES
        in_pmt = pmt.cons(pmt.intern('string'), pmt.from_long(1000))
        in_str = b'\x07\x02\x00\x06string\x03\x00\x00\x03\xe8'
        out_str = pmt.serialize_str(in_pmt)
        self.assertEqual(out_str, in_str)
        in_str = b'\x07\x07\x06\x06\x07\x00\x01'
        in_pmt = pmt.cons(pmt.PMT_EOF, pmt.cons(pmt.PMT_T, pmt.PMT_F))
        out_pmt = pmt.deserialize_str(in_str)
        self.assertTrue(pmt.equal(out_pmt, in_pmt))

        # dict SERDES
        d = pmt.make_dict()
        d = pmt.dict_add(d, pmt.intern('k0'), pmt.from_long(1))
        d = pmt.dict_add(d, pmt.intern('k1'), pmt.from_double(2.22222))
        d = pmt.dict_add(d, pmt.intern('k2'), pmt.from_long(3))
        in_str = b'\t\x07\x02\x00\x02k2\x03\x00\x00\x00\x03\t\x07\x02\x00\x02k1\x04@\x01\xc7\x1bG\x84#\x10\t\x07\x02\x00\x02k0\x03\x00\x00\x00\x01\x06'
        out_str = pmt.serialize_str(d)
        self.assertEqual(out_str, in_str)
        in_str = b'\t\x07\x02\x00\x03vec\n\x00\x00\x00\x00\x04\x01\x00\x01\x02\x03\x04\x06'
        d = pmt.dict_add(pmt.make_dict(), pmt.intern('vec'),
                         pmt.init_u8vector(4, [1, 2, 3, 4]))
        out_pmt = pmt.deserialize_str(in_str)
        self.assertTrue(pmt.equal(out_pmt, d))

    def test21_absolute_serialization_nums(self):
        # uint64 SERDES
        in_num = 9999876
        in_str = b'\x0b\x00\x00\x00\x00\x00\x98\x96\x04'
        out_str = pmt.serialize_str(pmt.from_uint64(in_num))
        self.assertEqual(out_str, in_str)
        in_str = b'\x0b\xff\xff\xff\xff\xff\xff\xff\xff'
        in_num = 0xffffffffffffffff
        out_num = pmt.to_uint64(pmt.deserialize_str(in_str))
        self.assertEqual(out_num, in_num)

        # long int SERDES
        in_num = 2432141
        in_str = b'\x03\x00%\x1c\x8d'
        out_str = pmt.serialize_str(pmt.from_long(in_num))
        self.assertEqual(out_str, in_str)
        in_str = b'\x03\xfdy\xe4\xb7'
        in_num = -42343241
        out_num = pmt.to_long(pmt.deserialize_str(in_str))
        self.assertEqual(out_num, in_num)

        # float SERDES
        in_num = -1.11
        in_str = b'\x04\xbf\xf1\xc2\x8f`\x00\x00\x00'
        out_str = pmt.serialize_str(pmt.from_float(in_num))
        self.assertEqual(out_str, in_str)
        in_str = b'\x04@\x8e\xdd;`\x00\x00\x00'
        in_num = 987.6539916992188
        out_num = pmt.to_float(pmt.deserialize_str(in_str))
        self.assertEqual(out_num, in_num)

        # double SERDES
        in_num = 987654.321
        in_str = b'\x04A.$\x0c\xa4Z\x1c\xac'
        out_str = pmt.serialize_str(pmt.from_double(in_num))
        self.assertEqual(out_str, in_str)
        in_str = b'\x04\xbf\xdb\x19\x84@A\r\xbc'
        in_num = -.42343241
        out_num = pmt.to_double(pmt.deserialize_str(in_str))
        self.assertEqual(out_num, in_num)

    def test22_absolute_serialization_int_uvecs(self):
        # u8_vector SERDES
        in_vec = [1, 3, 128, 255]
        in_str = b'\n\x00\x00\x00\x00\x04\x01\x00\x01\x03\x80\xff'
        out_str = pmt.serialize_str(pmt.init_u8vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x00\x00\x00\x00\x07\x01\x00\x02\x03\x04\t\n\x19@'
        in_vec = [2, 3, 4, 9, 10, 25, 64]
        out_vec = pmt.u8vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # s8_vector SERDES
        in_vec = [1, 3, 127, -128]
        in_str = b'\n\x01\x00\x00\x00\x04\x01\x00\x01\x03\x7f\x80'
        out_str = pmt.serialize_str(pmt.init_s8vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x01\x00\x00\x00\x07\x01\x00\x02\x00\x04\xf7\n\x19\xc0'
        in_vec = [2, 0, 4, -9, 10, 25, -64]
        out_vec = pmt.s8vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # u16_vector SERDES
        in_vec = [0xfffe, 0, 256, 0xffff]
        in_str = b'\n\x02\x00\x00\x00\x04\x01\x00\xff\xfe\x00\x00\x01\x00\xff\xff'
        out_str = pmt.serialize_str(pmt.init_u16vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x02\x00\x00\x00\x04\x01\x00\xff\xff\x00\x00\x00\x01\x00\x02'
        in_vec = [0xffff, 0, 1, 2]
        out_vec = pmt.u16vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # s16_vector SERDES
        in_vec = [0x7fff, 0, -256, -0x8000]
        in_str = b'\n\x03\x00\x00\x00\x04\x01\x00\x7f\xff\x00\x00\xff\x00\x80\x00'
        out_str = pmt.serialize_str(pmt.init_s16vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x03\x00\x00\x00\x05\x01\x00\x00\x01\x00\x02\x00\x03\x00\x04\xff\xfb'
        in_vec = [1, 2, 3, 4, -5]
        out_vec = pmt.s16vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # u32_vector SERDES
        in_vec = [0x01020304, 0x05060708, 0, 100000000]
        in_str = b'\n\x04\x00\x00\x00\x04\x01\x00\x01\x02\x03\x04\x05\x06\x07\x08\x00\x00\x00\x00\x05\xf5\xe1\x00'
        out_str = pmt.serialize_str(pmt.init_u32vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x04\x00\x00\x00\x06\x01\x00\x00\x00\x000\xff\xff\xff\xff\x00\x00\x00\x00\x00\x00\x00\t\x00\x00\x00\x0c\x00\x00\x10\x00'
        in_vec = [48, 0xffffffff, 0, 9, 12, 4096]
        out_vec = pmt.u32vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # s32_vector SERDES
        in_vec = [-0x0, -0x8000000, 1, 0x7ffffff]
        in_str = b'\n\x05\x00\x00\x00\x04\x01\x00\x00\x00\x00\x00\xf8\x00\x00\x00\x00\x00\x00\x01\x07\xff\xff\xff'
        out_str = pmt.serialize_str(pmt.init_s32vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x05\x00\x00\x00\x05\x01\x00\x00\x00\x000\x7f\xff\xff\xff\x00\x00\x00\x00\xff\xff\xff\xf7\xff\xff\xff\xf3'
        in_vec = [48, 0x7fffffff, 0, -9, -13]
        out_vec = pmt.s32vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # u64_vector SERDES NOT EXPOSED

        # i64_vector SERDES NOT EXPOSED

    def test23_absolute_serialization_float_uvecs(self):
        # f32_vector SERDES
        in_vec = [0x01020304, 2.1, 3.1415, 1e-9]
        # old serialization (f32 serialized as f64):
        # in_str = b'\n\x08\x00\x00\x00\x04\x01\x00Ap 0@\x00\x00\x00@\x00\xcc\xcc\xc0\x00\x00\x00@\t!\xca\xc0\x00\x00\x00>\x11.\x0b\xe0\x00\x00\x00'
        in_str = b'\n\x08\x00\x00\x00\x04\x01\x00K\x81\x01\x82@\x06ff@I\x0eV0\x89p_'
        out_str = pmt.serialize_str(pmt.init_f32vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        # old serialization (f32 serialized as f64):
        # in_str = b'\n\x08\x00\x00\x00\x04\x01\x00?\xf0\x00\x00\x00\x00\x00\x00@\x00\x00\x00\x00\x00\x00\x00\xbf\xc0\x00\x00\x00\x00\x00\x00@\xfe$\x00\x00\x00\x00\x00'
        in_str = b'\n\x08\x00\x00\x00\x04\x01\x00?\x80\x00\x00@\x00\x00\x00\xbe\x00\x00\x00G\xf1 \x00'
        in_vec = [1., 2., -0.125, 123456.0]
        out_vec = pmt.f32vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # f64_vector SERDES
        in_vec = [0x010203040506, 2.1, 1e-9]
        in_str = b'\n\t\x00\x00\x00\x03\x01\x00Bp 0@P`\x00@\x00\xcc\xcc\xcc\xcc\xcc\xcd>\x11.\x0b\xe8&\xd6\x95'
        out_str = pmt.serialize_str(pmt.init_f64vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\t\x00\x00\x00\x04\x01\x00?\xf0\x00\x00\x00\x00\x00\x00@\x00\x00\x00\x00\x00\x00\x00\xbf\xc0\x00\x00\x00\x00\x00\x00A\x9do4T\x00\x00\x00'
        in_vec = [1., 2., -0.125, 123456789.0]
        out_vec = pmt.f64vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # c32_vector SERDES
        in_vec = [0x01020304 - 1j, 3.1415 + 99.99j]
        # old serialization (c32 serialized as c64):
        # in_str = b'\n\n\x00\x00\x00\x02\x01\x00Ap 0@\x00\x00\x00\xbf\xf0\x00\x00\x00\x00\x00\x00@\t!\xca\xc0\x00\x00\x00@X\xff\\ \x00\x00\x00'
        in_str = struct.pack(">BBIBBffff", 0x0a, 0x0a, len(in_vec), 1, 0,
                             in_vec[0].real, in_vec[0].imag, in_vec[1].real, in_vec[1].imag)
        out_str = pmt.serialize_str(pmt.init_c32vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_vec = [1 + 1j, .125 - 9999999j]
        # old serialization (c32 serialized as c64):
        # in_str = b'\n\n\x00\x00\x00\x02\x01\x00?\xf0\x00\x00\x00\x00\x00\x00?\xf0\x00\x00\x00\x00\x00\x00?\xc0\x00\x00\x00\x00\x00\x00\xc1c\x12\xcf\xe0\x00\x00\x00'
        in_str = struct.pack(">BBIBBffff", 0x0a, 0x0a, len(in_vec), 1, 0,
                             in_vec[0].real, in_vec[0].imag, in_vec[1].real, in_vec[1].imag)
        out_vec = pmt.c32vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

        # c64_vector SERDES
        in_vec = [0xffffffff, .9j]
        in_str = b'\n\x0b\x00\x00\x00\x02\x01\x00A\xef\xff\xff\xff\xe0\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00?\xec\xcc\xcc\xcc\xcc\xcc\xcd'
        out_str = pmt.serialize_str(pmt.init_c64vector(len(in_vec), in_vec))
        self.assertEqual(out_str, in_str)
        in_str = b'\n\x0b\x00\x00\x00\x02\x01\x00?\xf0\x00\x00\x00\x00\x00\x00?\xf0\x00\x00\x00\x00\x00\x00?\xc0\x00\x00\x00\x00\x00\x00\xc1c\x12\xcf\xe0\x00\x00\x00'
        in_vec = [1 + 1j, .125 - 9999999j]
        out_vec = pmt.c64vector_elements(pmt.deserialize_str(in_str))
        self.assertEqual(out_vec, in_vec)

    def test23_none_exception(self):
        with self.assertRaises(TypeError):
            pmt.is_bool(None)

        with self.assertRaises(TypeError):
            pmt.is_true(None)

        with self.assertRaises(TypeError):
            pmt.is_false(None)

        with self.assertRaises(TypeError):
            pmt.to_bool(None)

        with self.assertRaises(TypeError):
            pmt.is_symbol(None)

        with self.assertRaises(TypeError):
            pmt.symbol_to_string(None)

        with self.assertRaises(TypeError):
            pmt.is_number(None)

        with self.assertRaises(TypeError):
            pmt.is_integer(None)

        with self.assertRaises(TypeError):
            pmt.to_long(None)

        with self.assertRaises(TypeError):
            pmt.is_uint64(None)

        with self.assertRaises(TypeError):
            pmt.to_uint64(None)

        with self.assertRaises(TypeError):
            pmt.is_real(None)

        with self.assertRaises(TypeError):
            pmt.to_double(None)

        with self.assertRaises(TypeError):
            pmt.to_float(None)

        with self.assertRaises(TypeError):
            pmt.is_complex(None)

        with self.assertRaises(TypeError):
            pmt.to_complex(None)

        with self.assertRaises(TypeError):
            pmt.is_null(None)

        with self.assertRaises(TypeError):
            pmt.is_pair(None)

        with self.assertRaises(TypeError):
            pmt.cons(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.cons(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dcons(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dcons(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.car(None)

        with self.assertRaises(TypeError):
            pmt.cdr(None)

        with self.assertRaises(TypeError):
            pmt.set_car(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.set_car(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.set_cdr(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.set_cdr(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.caar(None)

        with self.assertRaises(TypeError):
            pmt.cadr(None)

        with self.assertRaises(TypeError):
            pmt.cdar(None)

        with self.assertRaises(TypeError):
            pmt.cddr(None)

        with self.assertRaises(TypeError):
            pmt.caddr(None)

        with self.assertRaises(TypeError):
            pmt.cadddr(None)

        with self.assertRaises(TypeError):
            pmt.is_tuple(None)

        for tuple_len in range(1, 11):
            for i in range(tuple_len):
                args = [pmt.PMT_NIL] * tuple_len
                args[i] = None
                with self.assertRaises(TypeError):
                    pmt.make_tuple(*args)

        with self.assertRaises(TypeError):
            pmt.to_tuple(None)

        with self.assertRaises(TypeError):
            pmt.tuple_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.is_vector(None)

        with self.assertRaises(TypeError):
            pmt.make_vector(0, None)

        with self.assertRaises(TypeError):
            pmt.vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.vector_set(None, 0, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.vector_set(pmt.PMT_NIL, 0, None)

        with self.assertRaises(TypeError):
            pmt.vector_fill(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.vector_fill(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.is_blob(None)

        with self.assertRaises(TypeError):
            pmt.blob_data(None)

        with self.assertRaises(TypeError):
            pmt.blob_length(None)

        with self.assertRaises(TypeError):
            pmt.is_uniform_vector(None)

        with self.assertRaises(TypeError):
            pmt.is_u8vector(None)

        with self.assertRaises(TypeError):
            pmt.is_s8vector(None)

        with self.assertRaises(TypeError):
            pmt.is_u16vector(None)

        with self.assertRaises(TypeError):
            pmt.is_s16vector(None)

        with self.assertRaises(TypeError):
            pmt.is_u32vector(None)

        with self.assertRaises(TypeError):
            pmt.is_s32vector(None)

        with self.assertRaises(TypeError):
            pmt.is_u64vector(None)

        with self.assertRaises(TypeError):
            pmt.is_s64vector(None)

        with self.assertRaises(TypeError):
            pmt.is_f32vector(None)

        with self.assertRaises(TypeError):
            pmt.is_f64vector(None)

        with self.assertRaises(TypeError):
            pmt.is_c32vector(None)

        with self.assertRaises(TypeError):
            pmt.is_c64vector(None)

        with self.assertRaises(TypeError):
            pmt.uniform_vector_itemsize(None)

        with self.assertRaises(TypeError):
            pmt.u8vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.s8vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.u16vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.s16vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.u32vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.s32vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.u64vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.s64vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.f32vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.f64vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.c32vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.c64vector_ref(None, 0)

        with self.assertRaises(TypeError):
            pmt.u8vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.s8vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.u16vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.s16vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.u32vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.s32vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.u64vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.s64vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.f32vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.f64vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.c32vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.c64vector_set(None, 0, 0)

        with self.assertRaises(TypeError):
            pmt.uniform_vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u8vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s8vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u16vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s16vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u32vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s32vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u64vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s64vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.f32vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.f64vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.c32vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.c64vector_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u8vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.s8vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.u16vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.s16vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.u32vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.s32vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.u64vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.s64vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.f32vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.f64vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.c32vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.c64vector_elements(None)

        with self.assertRaises(TypeError):
            pmt.uniform_vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u8vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s8vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u16vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s16vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u32vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s32vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.u64vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.s64vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.f32vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.f64vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.c32vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.c64vector_writable_elements(None, 0)

        with self.assertRaises(TypeError):
            pmt.is_dict(None)

        with self.assertRaises(TypeError):
            pmt.dict_add(None, pmt.PMT_NIL, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_add(pmt.PMT_NIL, None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_add(pmt.PMT_NIL, pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dict_delete(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_delete(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dict_has_key(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_has_key(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dict_ref(None, pmt.PMT_NIL, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_ref(pmt.PMT_NIL, None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_ref(pmt.PMT_NIL, pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dict_items(None)

        with self.assertRaises(TypeError):
            pmt.dict_keys(None)

        with self.assertRaises(TypeError):
            pmt.dict_update(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.dict_update(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.dict_values(None)

        with self.assertRaises(TypeError):
            pmt.is_any(None)

        with self.assertRaises(TypeError):
            pmt.is_pdu(None)

        with self.assertRaises(TypeError):
            pmt.eq(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.eq(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.eqv(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.eqv(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.equal(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.equal(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.assq(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.assq(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.assv(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.assv(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.assoc(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.assoc(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.reverse(None)

        with self.assertRaises(TypeError):
            pmt.reverse_x(None)

        with self.assertRaises(TypeError):
            pmt.acons(None, pmt.PMT_NIL, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.acons(pmt.PMT_NIL, None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.acons(pmt.PMT_NIL, pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.nth(0, None)

        with self.assertRaises(TypeError):
            pmt.nthcdr(0, None)

        with self.assertRaises(TypeError):
            pmt.memq(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.memq(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.memv(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.memv(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.member(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.member(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.subsetp(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.subsetp(pmt.PMT_NIL, None)

        for list_len, func in enumerate((pmt.list1, pmt.list2, pmt.list3, pmt.list4, pmt.list5, pmt.list6), start=1):
            for i in range(list_len):
                args = [pmt.PMT_NIL] * list_len
                args[i] = None
                with self.assertRaises(TypeError):
                    func(*args)

        with self.assertRaises(TypeError):
            pmt.list_add(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.list_add(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.list_rm(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.list_rm(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.list_has(None, pmt.PMT_NIL)

        with self.assertRaises(TypeError):
            pmt.list_has(pmt.PMT_NIL, None)

        with self.assertRaises(TypeError):
            pmt.is_eof_object(None)

        with self.assertRaises(TypeError):
            pmt.write_string(None)

        with self.assertRaises(TypeError):
            pmt.print(None)

        with self.assertRaises(TypeError):
            pmt.length(None)

        with self.assertRaises(TypeError):
            pmt.serialize_str(None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, digital, dtv

import os
import pmt
from os.path import getsize
import numpy as np

ts_in_file = os.path.join(os.getenv("srcdir"), "vv009-4kfft.ts")
complex_out_file = os.path.join(os.getenv("srcdir"), "vv009-4kfft.cfile")


class test_dtv(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.outfile = "vv.cfile"

    def tearDown(self):
        self.tb = None
        os.remove(self.outfile)

    def test_000(self):
        infile = ts_in_file
        testfile = complex_out_file
        file_source = blocks.file_source(
            gr.sizeof_char * 1, infile, False, 0, 0)
        file_source.set_begin_tag(pmt.PMT_NIL)
        bbheader = dtv.dvb_bbheader_bb(
            dtv.STANDARD_DVBT2,
            dtv.FECFRAME_NORMAL,
            dtv.C2_3,
            dtv.RO_0_35,
            dtv.INPUTMODE_NORMAL,
            dtv.INBAND_OFF,
            168,
            4000000)
        bbscrambler = dtv.dvb_bbscrambler_bb(
            dtv.STANDARD_DVBT2,
            dtv.FECFRAME_NORMAL,
            dtv.C2_3)
        bch = dtv.dvb_bch_bb(
            dtv.STANDARD_DVBT2,
            dtv.FECFRAME_NORMAL,
            dtv.C2_3)
        ldpc = dtv.dvb_ldpc_bb(
            dtv.STANDARD_DVBT2,
            dtv.FECFRAME_NORMAL,
            dtv.C2_3,
            dtv.MOD_OTHER)
        interleaver = dtv.dvbt2_interleaver_bb(
            dtv.FECFRAME_NORMAL,
            dtv.C2_3,
            dtv.MOD_64QAM)
        modulator = dtv.dvbt2_modulator_bc(
            dtv.FECFRAME_NORMAL,
            dtv.MOD_64QAM,
            dtv.ROTATION_ON)
        cellinterleaver = dtv.dvbt2_cellinterleaver_cc(
            dtv.FECFRAME_NORMAL,
            dtv.MOD_64QAM,
            3,
            3)
        framemapper = dtv.dvbt2_framemapper_cc(
            dtv.FECFRAME_NORMAL,
            dtv.C2_3,
            dtv.MOD_64QAM,
            dtv.ROTATION_ON,
            3,
            3,
            dtv.CARRIERS_NORMAL,
            dtv.FFTSIZE_4K,
            dtv.GI_1_32,
            dtv.L1_MOD_16QAM,
            dtv.PILOT_PP7,
            2,
            8,
            dtv.PAPR_TR,
            dtv.VERSION_111,
            dtv.PREAMBLE_T2_SISO,
            dtv.INPUTMODE_NORMAL,
            dtv.RESERVED_OFF,
            dtv.L1_SCRAMBLED_OFF,
            dtv.INBAND_OFF)
        freqinterleaver = dtv.dvbt2_freqinterleaver_cc(
            dtv.CARRIERS_NORMAL,
            dtv.FFTSIZE_4K,
            dtv.PILOT_PP7,
            dtv.GI_1_32,
            8,
            dtv.PAPR_TR,
            dtv.VERSION_111,
            dtv.PREAMBLE_T2_SISO
        )
        pilotgenerator = dtv.dvbt2_pilotgenerator_cc(
            dtv.CARRIERS_NORMAL,
            dtv.FFTSIZE_4K,
            dtv.PILOT_PP7,
            dtv.GI_1_32,
            8,
            dtv.PAPR_TR,
            dtv.VERSION_111,
            dtv.PREAMBLE_T2_SISO,
            dtv.MISO_TX1,
            dtv.EQUALIZATION_OFF,
            dtv.BANDWIDTH_8_0_MHZ,
            4096
        )
        paprtr = dtv.dvbt2_paprtr_cc(
            dtv.CARRIERS_NORMAL,
            dtv.FFTSIZE_4K,
            dtv.PILOT_PP7,
            dtv.GI_1_32,
            8,
            dtv.PAPR_TR,
            dtv.VERSION_111,
            3.0,
            10,
            4096
        )
        ofdm_cyclic_prefixer = digital.ofdm_cyclic_prefixer(
            4096,
            4096 + 4096 // 32,
            0,
            '')
        p1insertion = dtv.dvbt2_p1insertion_cc(
            dtv.CARRIERS_NORMAL,
            dtv.FFTSIZE_4K,
            dtv.GI_1_32,
            8,
            dtv.PREAMBLE_T2_SISO,
            dtv.SHOWLEVELS_OFF,
            3.01
        )
        file_sink = blocks.file_sink(gr.sizeof_gr_complex * 1, self.outfile, False)
        file_sink.set_unbuffered(True)
        self.tb.connect(
            file_source,
            bbheader,
            bbscrambler,
            bch,
            ldpc,
            interleaver,
            modulator,
            cellinterleaver,
            framemapper,
            freqinterleaver,
            pilotgenerator,
            paprtr,
            ofdm_cyclic_prefixer,
            p1insertion,
            file_sink)
        self.tb.run()
        file_sink.close()

        self.assertEqual(getsize(self.outfile), getsize(testfile))

        out_data = np.fromfile(self.outfile, dtype=np.float32)
        expected_data = np.fromfile(testfile, dtype=np.dtype("<f"))

        self.assertFloatTuplesAlmostEqual(out_data, expected_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_dtv)



# Copyright 2021 Marcus Müller
# SPDX-License-Identifier: LGPL-3.0-or-later
class _clang_format_options:
    def __init__(self, clangfile=None):
        if not clangfile:
            clangfile = ".clang-format"
        self.lines = []
        with open(clangfile, encoding="utf-8") as opened:
            for line in opened:
                if line.strip().startswith("#"):
                    continue
                self.lines.append(line.rstrip().split(":"))

    def __getitem__(self, string):
        path = string.split(".")
        value = None
        for crumble in path:
            for line in self.lines:
                if line[0].strip() == crumble:
                    if len(line) > 1:
                        value = line[1].strip().rstrip()
                    break
        return value


_clang_format = _clang_format_options()

with section("parse"):
    additional_commands = {
        'gr_python_install': {
            'flags': [],
            'kwargs': {
                "PROGRAMS": "*",
                "FILES": "*",
                "DESTINATION": "*"
            }
        },
    }

with section("markup"):
    first_comment_is_literal = True
    enable_markup = False

with section("format"):
    disable = False
    line_width = int(_clang_format["ColumnLimit"])
    tab_size = int(_clang_format["IndentWidth"])
    min_prefix_chars = tab_size
    max_prefix_chars = 3 * tab_size
    use_tabchars = _clang_format["UseTab"] in ("ForIndentation",
                                               "ForContinuationAndIndentation",
                                               "Always")
    separate_ctrl_name_with_space = False
    separate_fn_name_with_space = False
    dangle_parens = False
    command_case = 'canonical'
    keyword_case = 'upper'

with section("lint"):
    max_arguments = 6
    max_localvars = 20
    max_statements = 75



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math


def sig_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [amp * math.cos(2. * math.pi * freq * x) for x in t]
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [amp * math.cos(2. * math.pi * freq * x) +
         1j * amp * math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_rms(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_ff(self):
        amp = 2
        src_data = sig_source_f(1, 0.01, amp, 200)
        N = 750000

        expected_data = amp / math.sqrt(2.0)

        src = blocks.vector_source_f(src_data, True)
        head = blocks.head(gr.sizeof_float, N)
        op = blocks.rms_ff(0.0001)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, head, op, dst)
        self.tb.run()

        dst_data = dst.data()
        self.assertAlmostEqual(dst_data[-1], expected_data, 4)

    def test_cf(self):
        amp = 4
        src_data = sig_source_c(1, 0.01, amp, 200)
        N = 750000

        expected_data = amp

        src = blocks.vector_source_c(src_data, True)
        head = blocks.head(gr.sizeof_gr_complex, N)
        op = blocks.rms_cf(0.0001)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, head, op, dst)
        self.tb.run()

        dst_data = dst.data()
        self.assertAlmostEqual(dst_data[-1], expected_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_rms)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt


class test_delay(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        delta_t = 0
        tb = self.tb
        src_data = [float(x) for x in range(0, 100)]
        expected_result = list(delta_t * [0.0] + src_data)

        src = blocks.vector_source_f(src_data)
        op = blocks.delay(gr.sizeof_float, delta_t)
        dst = blocks.vector_sink_f()

        tb.connect(src, op, dst)
        tb.run()
        dst_data = dst.data()
        self.assertEqual(expected_result, dst_data)

    def test_010(self):
        delta_t = 10
        tb = self.tb
        src_data = [float(x) for x in range(0, 100)]
        expected_result = list(delta_t * [0.0] + src_data)

        src = blocks.vector_source_f(src_data)
        op = blocks.delay(gr.sizeof_float, delta_t)
        dst = blocks.vector_sink_f()

        tb.connect(src, op, dst)
        tb.run()
        dst_data = dst.data()
        self.assertEqual(expected_result, dst_data)

    def test_020(self):
        tb = self.tb
        vector_sink = blocks.vector_sink_f(1)
        ref_sink = blocks.vector_sink_f(1)
        tags_strobe = blocks.tags_strobe(
            gr.sizeof_float * 1,
            pmt.intern("TEST"),
            100,
            pmt.intern("strobe"))
        head = blocks.head(gr.sizeof_float * 1, 10**5)
        delay = blocks.delay(gr.sizeof_float * 1, 100)
        tb.connect((delay, 0), (head, 0))
        tb.connect((head, 0), (vector_sink, 0))
        tb.connect((tags_strobe, 0), (delay, 0))
        tb.connect((tags_strobe, 0), (ref_sink, 0))
        tb.run()

        tags = vector_sink.tags()
        self.assertNotEqual(len(tags), 0)
        lastoffset = tags[0].offset - 100
        for tag in tags:
            newoffset = tag.offset
            self.assertEqual(newoffset, lastoffset + 100)
            lastoffset = newoffset


if __name__ == '__main__':
    gr_unittest.run(test_delay)
